# -*- coding: latin-1 -*-

import copy, time, StringIO
import unittest

from nive.utils.dataPool2.base import *
from nive.utils.dataPool2.connection import Connection
from nive.utils.dataPool2.structure import *

from nive.tests import __local

conf = {}
conf["root"] = __local.ROOT
conf["log"] = __local.ROOT+"sqllog.txt"
conf["codePage"] =    u"utf-8"
conf["dbCodePage"] = u"utf-8"
conf["backupVersions"] = 0
conf["useTrashcan"] = 0

conf["debug"] = 0


stdMeta = [u"id",
u"title",
u"pool_type",
u"pool_category",
u"pool_state",
u"pool_create",
u"pool_change",
u"pool_createdby",
u"pool_changedby",
u"pool_unitref",
u"pool_wfp",
u"pool_wfa",
u"pool_filename",
u"pool_stag",
u"pool_sort",
u"pool_datatbl",
u"pool_dataref"]

struct = {}
struct[u"pool_meta"] = stdMeta
struct[u"data1"] = (u"ftext",u"fnumber",u"fdate",u"flist",u"fmselect",u"funit",u"funitlist")
struct[u"data2"] = (u"fstr",u"ftext")

ftypes = {}
ftypes[u"data2"] = {u"fstr":"string" ,u"ftext":"text"}

SystemFlds = (
{"id": u"id",             "datatype": "number",    "size": 8,   "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1000, "name": "ID",},
{"id": u"title",          "datatype": "string",    "size": 255, "default": "",    "required": 0,     "readonly": 0, "settings": {}, "sort":  1100, "name": "Title"},
{"id": u"pool_type",      "datatype": "list",      "size": 35,  "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  1200, "name": "Type"},
{"id": u"pool_category",  "datatype": "list",      "size": 35,  "default": "",    "required": 0,     "readonly": 0, "settings": {}, "sort":  1300, "name": "Category"},
{"id": u"pool_filename",  "datatype": "string",    "size": 255, "default": "",    "required": 0,     "readonly": 0, "settings": {}, "sort":  1400, "name": "Filename"},
{"id": u"pool_create",    "datatype": "date",      "size": 0,   "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  1500, "name": "Created"},
{"id": u"pool_change",    "datatype": "datetime",  "size": 0,   "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  1600, "name": "Changed"},
{"id": u"pool_createdby", "datatype": "string",    "size": 35,  "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1700, "name": "Created by"},
{"id": u"pool_changedby", "datatype": "string",    "size": 35,  "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1800, "name": "Changed by"},
{"id": u"pool_wfp",       "datatype": "list",      "size": 35,  "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1900, "name": "Workflow Process"},
{"id": u"pool_wfa",       "datatype": "list",      "size": 35,  "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  2000, "name": "Workflow Activity"},
{"id": u"pool_state",     "datatype": "number",    "size": 4,   "default": 1,     "required": 0,     "readonly": 1, "settings": {}, "sort":  2100, "name": "State"},
{"id": u"pool_sort",      "datatype": "float",     "size": 8,   "default": 0.0,   "required": 0,     "readonly": 1, "settings": {}, "sort":  2200, "name": "Sort"},
{"id": u"pool_unitref",   "datatype": "number",    "size": 8,   "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  5000, "name": "Reference"},
{"id": u"pool_stag",      "datatype": "number",    "size": 4,   "default": 0,     "required": 0,     "readonly": 1, "settings": {}, "sort":  5100, "name": "Select Number"},
{"id": u"pool_datatbl",   "datatype": "string",    "size": 35,  "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  5200, "name": "Data Table Name"},
{"id": u"pool_dataref",   "datatype": "number",    "size": 8,   "default":  0,    "required": 1,     "readonly": 1, "settings": {}, "sort":  5200, "name": "Data Table Name"},
)
Fulltext = ("id","text","files")
Files = ("id","filename","path","size","extension","filekey","version")

    
# test data ----------------------------------------------------------------------
data1_1 = {u"ftext": "this is text!",
             u"fnumber": 123456,
             u"fdate": "2008/06/23 16:55",
             u"flist": "item 2",
             u"fmselect": ["item 1", "item 2", "item 3"],
             u"funit": 35,
             u"funitlist": [34, 35, 36]}
data2_1 = {u"fstr": u"this is sting!",
           u"ftext": u"this is text!"}
meta1 = {u"title":"title "}
file1_1 = "File 1 content, text text text text."
file1_2 = "File 2 content, text text text text."




class BaseTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_structure(self):
        base = Base()
        base.structure.Init(structure=struct, stdMeta=struct[u"pool_meta"])
        self.assert_(len(base.structure.stdMeta)==len(struct[u"pool_meta"]))

    def test_config(self):
        base = Base(**conf)
        self.assert_(str(base.GetRoot()).find(conf["root"])!=-1)

    def test_database(self):
        base = Base()
        try:
            base.SetConnection(Connection())
        except TypeError:
            pass
        base.usedconnection
        base.connection
        try:
            base.dbapi
        except ConnectionError:
            pass
            
    def test_sql(self):
        base = Base()
        base.structure.Init(structure=struct, stdMeta=struct[u"pool_meta"])
        try:
            base.SetConnection(Connection())
        except TypeError:
            pass
        #FmtSQLSelect(self, flds, parameter={}, dataTable = "", start = 0, max = 1000, **kw)
        flds = list(struct[u"pool_meta"])+list(struct[u"data1"])
        base.FmtSQLSelect(struct[u"pool_meta"])
        base.FmtSQLSelect(struct[u"pool_meta"], parameter={u"id":1})
        base.FmtSQLSelect(struct[u"pool_meta"], parameter={u"pool_type":u"data1"})
        base.FmtSQLSelect(flds, parameter={u"pool_type":u"data1"}, dataTable=u"data1")
        base.FmtSQLSelect(list(struct[u"data1"])+[u"-count(*)"], parameter={u"id":123}, dataTable=u"data1", singleTable=1)
        base.FmtSQLSelect(flds,
                          parameter={u"pool_type":u"data1"},
                          dataTable=u"data1",
                          operators={u"pool_type":u">"})
        base.FmtSQLSelect(flds,
                          parameter={u"pool_type":u"data1"},
                          dataTable=u"data1",
                          operators={u"pool_type":u"="},
                          start=3,
                          max=20,
                          ascending=1,
                          sort=u"title, pool_type")
        base.FmtSQLSelect(flds,
                          parameter={u"pool_type":u"data1",u"id":[1,2,3,4,5,6,7,8,9,0]},
                          dataTable=u"data1",
                          operators={u"pool_type":u"=", u"id":u"IN"},
                          jointype=u"LEFT",
                          logicalOperator=u"or",
                          condition=u"meta__.id > 2",
                          join=u"INNER JOIN table1 on (meta__.id=table1.id)",
                          groupby=u"pool_type",
                          start=3,
                          max=20,
                          sort=u"title, pool_type")
        base.GetFulltextSQL(u"test",
                            flds,
                          parameter={u"pool_type":u"data1",u"id":[1,2,3,4,5,6,7,8,9,0]},
                          dataTable=u"data1",
                          operators={u"pool_type":u"=", u"id":u"IN"},
                          jointype=u"LEFT",
                          logicalOperator=u"or",
                          condition=u"meta__.id > 2",
                          join=u"INNER JOIN table1 on (meta__.id=table1.id)",
                          groupby=u"pool_type",
                          start=3,
                          max=20,
                          sort=u"title, pool_type")







if __name__ == '__main__':
    unittest.main()




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Extension modules to store configuration values edited through the web interface.
Several backends are supported.

The functions only stores and loads the values passed to ``Save(values)``, not all configuration values.
Remaining values are loaded from python and configuration files in the file system.

For configuration storage and reference ``configuration.uid()`` is used as key.  
"""
import pickle
import time

from nive.definitions import implements, IPersistent, ModuleConf, Conf, IModuleConf
from nive.definitions import OperationalError, ProgrammingError





class PersistentConf(object):
    """
    configuration persistence base class ---------------------------------------
    """
    implements(IPersistent)
    
    def __init__(self, app, configuration):
        self.app = app
        self.conf = configuration
        
    def Load(self):
        """
        Load configuration values from backend and map to configuration.
        """
        raise TypeError, "subclass"
        
    def Save(self, values):
        """
        Store configuration values in backend.
        """
        raise TypeError, "subclass"
        
    def Changed(self):
        """
        Validate configuration and backend timestamp and check if 
        values have changed.
        """
        return False
        
    def _GetUid(self):
        return self.conf.uid()



        
def LoadStoredConfValues(app, pyramidConfig):
    # lookup persistent manager for configuration
    storage = app.Factory(IModuleConf, "persistence")
    if not storage:
        return
    try:
        db = app.NewDBApi()
        if not db:
            return
    except:
        return
    # adapters
    for conf in app.registry.registeredAdapters():
        storage(app=app, configuration=conf.factory).Load(db=db)
    for conf in app.registry.registeredUtilities():
        storage(app=app, configuration=conf.component).Load(db=db)
    db.close()
    

class DbPersistence(PersistentConf):
    """
    Stores configuration values in the configured databases' pool_sys table.
    """

    def Load(self, db=None):
        """
        Load configuration values from backend and map to configuration.
        Uses a raw database connection to access the database to allow being called 
        before startup. 
        """
        try:
            close = 0
            if not db:
                close = 1
                db = self.app.NewDBApi()
            sql = """select value,ts from pool_sys where id=%s""" % (db.placeholder)
            c=db.cursor()
            c.execute(sql, (self._GetUid(),))
            data = c.fetchall()
            c.close()
        except OperationalError:
            data = None
            db.rollback()
        except ProgrammingError: 
            data = None
            db.rollback()
        except Exception, e:
            # !!! log error and continue
            return None
        if close:
            db.close()
        if data:
            try:
                values = pickle.loads(data[0][0])
            except KeyError:
                # Invalid data
                return None
            lock = 0
            if self.conf.locked:
                lock = 1
                self.conf.unlock()
            self.conf.timestamp = data[0][1]
            #opt
            #for f in values.items():
            #    self.conf[f[0]] = f[1]
            self.conf.update(values)
            if lock:
                self.conf.lock()
            return values
        return None
        
    def Save(self, values, db=None):
        """
        Store configuration values in backend.
        Uses the datapool class to access the database.
        """
        ts = time.time()
        try:
            close = 0
            if not db:
                close = 1
                db = self.app.db
            sql = """select ts from pool_sys where id=%s""" % (db.placeholder)
            r = db.Query(sql, (self._GetUid(),))
            data = pickle.dumps(values)
            if len(r):
                db.UpdateFields("pool_sys", self._GetUid(), {"value":data,"ts":ts})
            else:
                db.InsertFields("pool_sys", {"value":data,"ts":ts, "id":self._GetUid()})
            db.Commit()
        except OperationalError: 
            data = None
            db.Undo()
        except ProgrammingError: 
            data = None
            db.Undo()
        if close:
            db.Close()
        lock = 0
        if self.conf.locked:
            lock = 1
            self.conf.unlock()
        self.conf.timestamp = ts
        #for f in values.items():
        #    self.conf[f[0]] = f[1]
        self.conf.update(values)
        if lock:
            self.conf.lock()
        return True
        
    def Changed(self):
        """
        Validate configuration timestamp and backend timestamp and check if 
        updates have occured.
        """
        return False


dbPersistenceConfiguration = ModuleConf(
    id = "persistence",
    name = u"Configuration persistence extension",
    context = DbPersistence,
    events = (Conf(event="finishRegistration", callback=LoadStoredConfValues),),
)




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Editor Root
------------
In edit mode the cms will use this root to render the webpage. nive.cms.root is used
in normal mode. 
"""

from nive.i18n import _
from nive.definitions import RootConf, implements, IWebsiteRoot, ICMSRoot
from nive.components.objects.base import PageRootBase


class cmsroot(PageRootBase):

    #implements(IWebsiteRoot, ICMSRoot)
    implements(ICMSRoot)
    extension = u"html"
    
    


# Root definition ------------------------------------------------------------------
#@nive_module
configuration = RootConf(
    id = "editor",
    context = "nive.cms.cmsview.cmsroot.cmsroot",
    template = "root.pt",
    default = False,
    subtypes = "*",
    name = _(u"Home"),
    extensions = ("nive.components.extensions.persistentRoot.Persistent",),
    description = __doc__
)



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__version__ = "0.9.11"
__doc__ = """nive"""


"""
Use `nive.definitions` to import Interfaces or Configuration classes
"""
from nive.utils.dataPool2.files import File
from nive.utils.dataPool2.base import FileNotFound


# -*- coding: utf-8 -*-

import time
import unittest

from nive.cms.tests.db_app import *

from nive.cms.design.view import *

from pyramid.httpexceptions import HTTPNotFound, HTTPFound, HTTPOk, HTTPForbidden
from pyramid import testing 
from pyramid.renderers import render



class tDesign(unittest.TestCase):

    def setUp(self):
        request = testing.DummyRequest()
        request._LOCALE_ = "en"
        self.config = testing.setUp(request=request)
        self.request = request
        self.app = app()
        self.app.Startup(self.config)        
        self.root = self.app.root()
        user = User(u"test")
        user.groups.append("group:editor")
        self.page = create_page(self.root, user)
        self.request.context = self.page

    def tearDown(self):
        user = User(u"test")
        testing.tearDown()
        root = self.app.root()
        self.root.Delete(self.page.id, user=user)
        self.app.Close()


    def test_functions(self):
        view = Design(self.root, self.request)
        view.HtmlTitle()
        view.IsEditmode()
        self.assertRaises(HTTPFound, view.app)
        self.assertRaises(HTTPNotFound, view.open)

    
    def test_views1(self):
        # root
        view = Design(self.root, self.request)
        view.HtmlTitle()
        view.index_tmpl()
        view.view()
        view.navigationTop(addHome=1)
        view.navigationTop(addHome=0)
        view.navigationTree(addHome=1, page=None)
        view.navigationTree(addHome=0, page=None)
        view.navigationTree(addHome=1, page=self.page)
        view.navigationSub(page=None)
        view.navigationSub(page=self.page)
        view.breadcrumbs(addHome=0)
        view.breadcrumbs(addHome=1)
        view.doc()
    

    def test_views2(self):
        # page
        view = Design(self.page, self.request)
        view.HtmlTitle()
        view.index_tmpl()
        view.view()
        view.navigationTop(addHome=1)
        view.navigationTop(addHome=0)
        view.navigationTree(addHome=1, page=None)
        view.navigationTree(addHome=0, page=None)
        view.navigationSub(page=None)
        view.breadcrumbs(addHome=0)
        view.breadcrumbs(addHome=1)
        
        
    def test_templates(self):
        user = User(u"test")
        user.groups.append("group:editor")
        view = Design(self.page, self.request)
        vrender = {"context":self.page, "view":view, "request": self.request, "cmsview":None}
        
        render("nive.cms.design:templates/page.pt", vrender)
        render("nive.cms.design:templates/root.pt", {"context":self.page.root(), "view":view, "cmsview":None})

        b1 = create_box(self.page, user)
        render("nive.cms.design:templates/box.pt", {"context":b1, "view":view})
        b2 = create_column(self.page, user)
        render("nive.cms.design:templates/column.pt", {"context":b2, "view":view})
        b3 = create_file(self.page, user)
        render("nive.cms.design:templates/file.pt", {"context":b3, "view":view})
        b4 = create_image(self.page, user)
        render("nive.cms.design:templates/image.pt", {"context":b4, "view":view})
        b5 = create_media(self.page, user)
        render("nive.cms.design:templates/media.pt", {"context":b5, "view":view})
        b6 = create_note(self.page, user)
        render("nive.cms.design:templates/note.pt", {"context":b6, "view":view})
        b7 = create_text(self.page, user)
        render("nive.cms.design:templates/text.pt", {"context":b7, "view":view})
        b8 = create_spacer(self.page, user)
        render("nive.cms.design:templates/spacer.pt", {"context":b8, "view":view})
        b9 = create_link(self.page, user)
        render("nive.cms.design:templates/link.pt", {"context":b9, "view":view})
        b0 = create_menublock(self.page, user)
        render("nive.cms.design:templates/menublock.pt", {"context":b0, "view":view, "request": self.request})
        b10 = create_code(self.page, user)
        render("nive.cms.design:templates/code.pt", {"context":b10, "view":view})
        b11 = create_news(self.page, user)
        render("nive.cms.design:templates/news.pt", {"context":b11, "view":view})
        
        # add to box
        b3 = create_file(b1, user)
        b4 = create_image(b1, user)
        b5 = create_media(b1, user)
        b6 = create_note(b1, user)
        b7 = create_text(b1, user)
        b8 = create_spacer(b1, user)
        b9 = create_link(b1, user)
        b0 = create_menublock(b1, user)
        b10 = create_code(b1, user)
        b11 = create_news(b1, user)
        render("nive.cms.design:templates/box.pt", {"context":b1, "view":view})
        
        # add to column
        b3 = create_file(b2, user)
        b4 = create_image(b2, user)
        b5 = create_media(b2, user)
        b6 = create_note(b2, user)
        b7 = create_text(b2, user)
        b8 = create_spacer(b2, user)
        b9 = create_link(b2, user)
        b0 = create_menublock(b2, user)
        b10 = create_code(b2, user)
        b11 = create_news(b2, user)
        render("nive.cms.design:templates/column.pt", {"context":b2, "view":view})
        
        # render page with elements
        render("nive.cms.design:templates/page.pt", {"context":self.page, "view":view, "cmsview":None})
        render("nive.cms.design:templates/root.pt", {"context":self.page.root(), "view":view, "cmsview":None})
        r=view.view()
        self.assertEqual(r.status_int, 200)
        self.assertGreater(r.content_length, 2000)
        
        # search
        self.request.POST = {"phrase": "a"}
        r=view.search()
        self.assertEqual(r.status_int, 200)
        self.assertGreater(r.content_length, 2000)
        
        

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import unittest

from nive.utils import country_data
from nive.utils import language


class Country_data(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    
    def test_defs(self):
        for ccc in country_data.countries:
            self.assert_(ccc[0], ccc)
            self.assert_(ccc[1], ccc)
            self.assert_(ccc[2], ccc)
            self.assert_(ccc[3], ccc)
                        
        
    def test_codes(self):
        for conf in country_data.GetCountries():
            self.assert_(conf.get("id"), conf)
            self.assert_(conf.get("name"), conf)
            
            
    def test_get(self):
        self.assert_(country_data.GetConf(u"DEU").get(u"code2")==u"DE")
        self.assert_(country_data.GetConf(u"DE").get(u"code")==u"DEU")
        self.assert_(country_data.GetConf(u"XX").get(u"code2")==None)
                        
        


class Country(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    
    def test_lang(self):
        cext = language.CountryExtension()
        for c in cext.Codelist():
            self.assert_(c.get("id"))
            self.assert_(c.get("name"))
            
    def test_conf(self):
        lext = language.CountryExtension()
        for c in country_data.countries:
            conf = lext.GetConf(c[0])
            self.assert_(conf.get("code"), c)
            self.assert_(conf.get("code2"), c)
            self.assert_(conf.get("name"), c)
            

    def test_name(self):
        lext = language.CountryExtension()
        for c in country_data.countries:
            name = lext.GetName(c[0])
            self.assert_(name, c)
            
            
            
            
            


# -*- coding: utf-8 -*-

import time
import unittest

from nive.workflow import WorkflowNotAllowed
from nive.security import Allow, Deny, Authenticated, Everyone

from nive.cms.tests.db_app import *

class ObjectTest(unittest.TestCase):

    def setUp(self):
        self.app = app()
        self.remove = []
        
    def tearDown(self):
        u = User(u"test")
        u.groups.append("group:editor")
        root = self.app.root("editor")
        for r in self.remove:
            root.Delete(r, u)
        self.app.Close()
        pass

    def test_root(self):
        a=self.app
        ccc = a.db.GetCountEntries()
        r=root(a)
        user = User(u"test")
        user.groups.append("group:editor")
        # add to root
        p = create_page(r, user=user)
        self.assert_(p)
        self.remove.append(p.id)
        b0 = create_menublock(r, user=user)
        self.assert_(b0)
        self.remove.append(b0.id)
        b1 = create_box(r, user=user)
        self.assert_(b1)
        self.remove.append(b1.id)
        col = r.GetColumn("left")
        if col:
            r.Delete(col.id, user=user)
        b2 = create_column(r, user=user)
        self.assert_(b2)
        self.remove.append(b2.id)
        b3 = create_file(r, user=user)
        self.assert_(b3)
        self.remove.append(b3.id)
        b4 = create_image(r, user=user)
        self.assert_(b4)
        self.remove.append(b4.id)
        b5 = create_media(r, user=user)
        self.assert_(b5)
        self.remove.append(b5.id)
        b6 = create_note(r, user=user)
        self.assert_(b6)
        self.remove.append(b6.id)
        b7 = create_text(r, user=user)
        self.assert_(b7)
        self.remove.append(b7.id)
        b8 = create_spacer(r, user=user)
        self.assert_(b8)
        self.remove.append(b8.id)
        b9 = create_link(r, user=user)
        self.assert_(b9)
        self.remove.append(b9.id)
        b10 = create_code(r, user=user)
        self.assert_(b10)
        self.remove.append(b10.id)
        r.Delete(p.id, user=user, obj=p)
        r.Delete(b0.id, user=user)
        r.Delete(b1.id, user=user)
        r.Delete(b2.id, user=user)
        r.Delete(b3.id, user=user)
        r.Delete(b4.id, user=user)
        r.Delete(b5.id, user=user)
        r.Delete(b6.id, user=user)
        r.Delete(b7.id, user=user)
        r.Delete(b8.id, user=user)
        r.Delete(b9.id, user=user)
        r.Delete(b10.id, user=user)
        self.assertEqual(ccc, a.db.GetCountEntries(), "Delete failed")
        
        # workflow failure add to root
        user.groups = ["group:looser"]
        self.assertRaises(WorkflowNotAllowed, create_page, r, user)


    def test_page(self):
        a=self.app
        ccc = a.db.GetCountEntries()
        r=root(a)
        user = User(u"test")
        user.groups.append("group:editor")
        # add to root
        p = create_page(r, user=user)
        self.assert_(p)
        self.remove.append(p.id)
        
        self.assert_(p.IsLinked()==u"")
        self.assert_(p.IsPage())
        p.meta["pool_groups"] = ["authenticated","another"]
        p.Init()
        self.assert_((Allow, Authenticated, "view") in p.__acl__)

        r = p
        b0 = create_menublock(r, user=user)
        self.assert_(b0)
        b1 = create_box(r, user=user)
        self.assert_(b1)
        b2 = create_column(r, user=user)
        self.assert_(b2)
        b3 = create_file(r, user=user)
        self.assert_(b3)
        b4 = create_image(r, user=user)
        self.assert_(b4)
        b5 = create_media(r, user=user)
        self.assert_(b5)
        b6 = create_note(r, user=user)
        self.assert_(b6)
        b7 = create_text(r, user=user)
        self.assert_(b7)
        b8 = create_spacer(r, user=user)
        self.assert_(b8)
        b9 = create_link(r, user=user)
        self.assert_(b9)
        b10 = create_code(r, user=user)
        self.assert_(b10)
        root(a).Delete(r.id, user=user, obj=r)
        self.assertEqual(ccc, a.db.GetCountEntries(), "Delete failed")
        
    def test_container(self):
        a=self.app
        ccc = a.db.GetCountEntries()
        r=root(a)
        user = User(u"test")
        user.groups.append("group:editor")
        # add to root
        b1 = create_box(r, user=user)
        self.assert_(b1)
        self.remove.append(b1.id)
        r = b1
        b3 = create_file(r, user=user)
        self.assert_(b3)
        b4 = create_image(r, user=user)
        self.assert_(b4)
        b5 = create_media(r, user=user)
        self.assert_(b5)
        b6 = create_note(r, user=user)
        self.assert_(b6)
        b7 = create_text(r, user=user)
        self.assert_(b7)
        b8 = create_spacer(r, user=user)
        self.assert_(b8)
        b9 = create_link(r, user=user)
        self.assert_(b9)
        b0 = create_menublock(r, user=user)
        self.assert_(b0)
        b10 = create_code(r, user=user)
        self.assert_(b10)
        
        r=root(a)
        col = r.GetColumn("left")
        if col:
            r.Delete(col.id, user=user)
        b2 = create_column(r, user=user)
        self.assert_(b2)
        self.remove.append(b2.id)
        r = b2
        b3 = create_file(r, user=user)
        self.assert_(b3)
        b4 = create_image(r, user=user)
        self.assert_(b4)
        b5 = create_media(r, user=user)
        self.assert_(b5)
        b6 = create_note(r, user=user)
        self.assert_(b6)
        b7 = create_text(r, user=user)
        self.assert_(b7)
        b8 = create_spacer(r, user=user)
        self.assert_(b8)
        b9 = create_link(r, user=user)
        self.assert_(b9)
        b0 = create_menublock(r, user=user)
        self.assert_(b0)
        b10 = create_code(r, user=user)
        self.assert_(b10)
        root(a).Delete(b1.id, user=user, obj=b1)
        root(a).Delete(b2.id, user=user, obj=b2)
        self.assertEqual(ccc, a.db.GetCountEntries(), "Delete failed")

    def test_objs(self):
        a=self.app
        ccc = a.db.GetCountEntries()
        r=root(a)
        user = User(u"test")
        user.groups.append("group:editor")
        p = create_page(r, user=user)
        self.remove.append(p.id)
        r = p
        #box
        b1 = create_box(r, user=user)
        b1.IsContainer()
        b1.GetPage()
        b1.GetElementContainer()
        b1.GetContainer()
        #column
        b2 = create_column(r, user=user)
        self.assert_(b2.IsLocal(r))
        b2.GetName()
        b2.IsContainer()
        b2.GetPage()
        b2.GetPages()
        b2.GetElementContainer()
        b2.GetContainer()
        self.assert_(b2.GetColumn("left")==b2)
        #file
        b3 = create_file(r, user=user)
        b3.GetDownloadTitle()
        b3.FilenameToTitle()
        #menublock
        b0 = create_menublock(r, user=user)
        b0.GetMenuPages()
        
        root(a).Delete(r.id, user=user)
        self.assertEqual(ccc, a.db.GetCountEntries(), "Delete failed")
        




if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------
__doc__ = """
Administration interface extensions

"""

from pyramid.renderers import render

from nive.i18n import _
from nive.definitions import ViewConf, ViewModuleConf, FieldConf, WidgetConf, Conf
from nive.definitions import IApplication, IAdminWidgetConf   
from nive.adminview.view import AdminBasics, ConfigurationForm

# view module definition ------------------------------------------------------------------

#@nive_module
configuration = ViewModuleConf(
    id = "cms_admin",
    name = _(u"CMS Administration"),
    context = IApplication,
    view = "nive.cms.cmsview.admin.CMSAdminView",
    templates = "nive.adminview:",
    permission = "administration"
)

t = configuration.templates
configuration.views = [
    ViewConf(name = "design",   attr = "design",     renderer = t+"form.pt"),
    ViewConf(name = "help",     attr = "help",       renderer = t+"help.pt", permission="read"),
]

configuration.widgets = [
    WidgetConf(name=_(u"Design"), viewmapper="design", id="admin.design", sort=1500,   apply=(IApplication,), widgetType=IAdminWidgetConf,
               description=u""),
    WidgetConf(name=_(u"Help"),                 viewmapper="help",   id="admin.help",   sort=50000,  apply=(IApplication,), widgetType=IAdminWidgetConf,
               description=_(u"Help and documentation.")),
]




class CMSAdminView(AdminBasics):
    
    def doc(self, template=u"default.pt"):
        return render(u"nive.cms:doc/"+template, {u"context":self.context, u"view":self, u"request": self.request}, request=self.request)
    
    def design(self):
        fields = (
            FieldConf(id=u"columns", datatype="lines",  size=100,  required=0, name=_(u"Column names"), description=_(u"Column names used in the main template. For multiple columns enter one per line.")),
        )
        form = ConfigurationForm(view=self, context=self.context.configuration, app=self.context)
        form.fields = fields
        form.Setup() 
        # process and render the form.
        result, data, action = form.Process()
        return {u"content": data, u"result": result, u"head": form.HTMLHead()}
    
    def help(self):
        return {}
    




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Object based events
-------------------
Events are object based and registered with the object itself. So they will
only be called as long as the object exists and are meant in the first place 
to be used in conjunction with object extension. 

Events work quite simple: the signal representing the event is just a string
and is registered in combination with a function. If the event is fired 
somewhere, the linked function will be called with event specific paramaters.

*signals* have not to be registered. They are just fired by calling *Signal(signal)*.

Every object and root fires the *init* event linked to **Init()** on creation.
So it is not necessary to register for *init* yourself and but use it as plugin point to 
register your own events.

For example if you would like to get your function **Edit()** called if the object is 
updated, add this function to your objects class: ::
    
    class myObject:
    
        def Init(self):
            self.ListenEvent("update", "Edit")
        
    
        def Edit(self, data):
            # e.g. process data 
            return
            
Inventing custom events is quite easy: :: 

        def SomeEvent(self):
            self.Signal("some_event")
            
            
For non Object or Root derived classes initialize the event system by adding a signal
to the __init__ function: ::

    def __init__(self):
        self.Signal("init")
    
"""

import weakref

class Events(object):
    
    def ListenEvent(self, signal, function):
        """
        Register a function for an event. 
        
        - *signal*: the event to listen for
        - *function*: callback to be called if the event is fired

        """
        if not self._eventdispatch.has_key(signal):
            self._eventdispatch[signal] = [function]
        else:
            self._eventdispatch[signal].append(function)


    def RemoveListener(self, signal, function):
        """
        Remove the function from an event.

        - *signal*: the event to listen for
        - *function*: callback to be called if the event is fired

        """
        if not self._eventdispatch.has_key(signal):
            return
        try:
            self._eventdispatch[signal].remove(function)
        except:
            pass
    

    def Signal(self, signal, raiseExcp=True, **kw):
        """
        Fire an event *signal*. *kw* are the parameters passed to the callback.
        """
        if signal==u"init":
            self.InitEvents()
            #return
        if not self._eventdispatch.has_key(signal):
            return
        if raiseExcp:
            for fnc in self._eventdispatch[signal]:
                if isinstance(fnc, basestring):
                    for cls in self.__class__.__mro__:
                        f = cls.__dict__.get(fnc)
                        if f != None:
                            f(self, **kw)
                else:
                    try:
                        fnc(context=self,**kw)
                    except TypeError:
                        fnc(**kw)
        else:
            for fnc in self._eventdispatch[signal]:
                try:
                    if isinstance(fnc, basestring):
                        for cls in self.__class__.__mro__:
                            f = cls.__dict__.get(fnc)
                            if f != None:
                                f(self, **kw)
                    else:
                        try:
                            fnc(context=self,**kw)
                        except TypeError:
                            fnc(**kw)
                except Exception, e:
                    pass


    def InitEvents(self):
        """
        Call Init() for every super class
        """
        if not hasattr(self, "_eventdispatch"):
            self._eventdispatch = {}
        for cls in self.__class__.__mro__:
            f = cls.__dict__.get("Init")
            if f != None:
                f(self)



    # bw 0.9.9. renamed functions
    def RegisterEvent(self, signal, function):
        return self.ListenEvent(signal, function)

    def RemoveEvent(self, signal, function):
        return self.RemoveListener(signal, function)





import unittest

class TestForm(unittest.TestCase):
    def _makeOne(self, schema, **kw):
        from nive.components.reform.form import Form
        return Form(schema, **kw)

    def test_ctor_buttons_strings(self):
        from nive.components.reform.widget import FormWidget
        schema = DummySchema()
        form = self._makeOne(schema, renderer='abc', action='action',
                             method='method', buttons=('button',),
                             formid='formid')
        self.assertEqual(form.schema, schema)
        self.assertEqual(form.renderer, 'abc')
        self.assertEqual(form.action, 'action')
        self.assertEqual(form.method, 'method')
        self.assertEqual(form.formid, 'formid')
        self.assertEqual(form.widget.__class__, FormWidget)
        self.assertEqual(len(form.buttons), 1)
        button = form.buttons[0]
        self.assertEqual(button.name, 'button')
        self.assertEqual(button.value, 'button')
        self.assertEqual(button.title, 'Button')

    def test_ctor_buttons_notstrings(self):
        from nive.components.reform.widget import FormWidget
        schema = DummySchema()
        form = self._makeOne(schema, renderer='abc', action='action',
                             method='method', buttons=(None,), formid='formid')
        self.assertEqual(form.schema, schema)
        self.assertEqual(form.renderer, 'abc')
        self.assertEqual(form.action, 'action')
        self.assertEqual(form.method, 'method')
        self.assertEqual(form.formid, 'formid')
        self.assertEqual(form.widget.__class__, FormWidget)
        self.assertEqual(len(form.buttons), 1)
        button = form.buttons[0]
        self.assertEqual(button, None)

    def test_ctor_passes_unknown_kwargs(self):
        schema = DummySchema()
        schema.children = [DummySchema()]
        form = self._makeOne(schema, a='a', b='b')
        self.assertEqual(form.schema, schema)
        self.assertEqual(form.a, 'a')
        self.assertEqual(form.b, 'b')
        child = form.children[0]
        self.assertEqual(child.a, 'a')
        self.assertEqual(child.b, 'b')


class TestIssues(unittest.TestCase):

    def test_issue_54(self):
        from nive.components import reform
        class LoginForm(reform.schema.Schema):
            username = reform.schema.SchemaNode(reform.schema.String())
            password = reform.schema.SchemaNode(reform.schema.String(),
                                           widget=reform.widget.PasswordWidget())
        def validate_password(node, d):
            raise reform.schema.Invalid(node,
                                   'Username does not match password')
        loginform = LoginForm(validator=validate_password)
        data = [('username', 'kai'),
                ('password', '123')]
        try:
            reform.Form(loginform).validate(data)
        except reform.ValidationFailure, e:
            rendered = e.render()
            self.assertTrue(
                'Username does not match password' in rendered
            )

class TestButton(unittest.TestCase):
    def _makeOne(self, **kw):
        from nive.components.reform.form import Button
        return Button(**kw)

    def test_ctor_title_None(self):
        button = self._makeOne()
        self.assertEqual(button.title, 'Submit')

    def test_ctor_type_None(self):
        button = self._makeOne()
        self.assertEqual(button.type, 'submit')

    def test_ctor_value_None(self):
        button = self._makeOne()
        self.assertEqual(button.value, 'submit')

    def test_name_with_space(self):
        button = self._makeOne(name="log\tin as a user")
        self.assertEquals(button.name, 'log_in_as_a_user')
        self.assertEquals(button.value, 'log_in_as_a_user')

    def test_ctor(self):
        button = self._makeOne(name='name', title='title',
                               type='type', value='value', disabled=True)
        self.assertEqual(button.value, 'value')
        self.assertEqual(button.name, 'name')
        self.assertEqual(button.title, 'title')
        self.assertEqual(button.type, 'type')
        self.assertEqual(button.disabled, True)

class TestRaw(unittest.TestCase):
    def _makeOne(self, val):
        from nive.components.reform.form import Raw
        return Raw(val)

    def test___html__(self):
        inst = self._makeOne('abc')
        self.assertEqual(inst.__html__(), 'abc')

class DummySchema(object):
    typ = None
    name = 'name'
    title = 'title'
    description = 'description'
    required = True
    children = ()
    default = 'default'
    sdefault = 'sdefault'
    def __init__(self, exc=None):
        self.exc = exc



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

import string

from pyramid.i18n import get_localizer
from pyramid.threadlocal import get_current_request

from nive.definitions import _
from nive.tools import Tool
from nive.helper import FakeLocalizer

from nive.utils.utils import ConvertToNumberList
from nive.utils.utils import ConvertHTMLToText
from nive.definitions import implements, Interface, ModuleConf, ToolConf, Conf 
from nive.definitions import IApplication, IPage, IRoot

class IFulltext(Interface):
    pass


class PageFulltext(object):
    """
    Fulltext support for cms pages. Automatically updates fulltext on commit. 
    """
    implements(IFulltext)

    def Init(self):
        self.ListenEvent("commit", "UpdateFulltext")
        

    # Fulltext ------------------------------------------

    def UpdateFulltext(self):
        """
        Update fulltext for entry. Text is generated automatically.
        """
        if not self.app.configuration.fulltextIndex:
            return
        if IPage.providedBy(self):
            # get text from contained elements
            text = [self.GetTexts()]
            for e in self.GetPageElements(addBoxContents=1, addColumnContents=1):
                text.append(e.GetTexts())
            self.dbEntry.WriteFulltext(self.FormatFulltext(text))
        elif IPage.providedBy(self) or IRoot.providedBy(self):
            # get text from contained elements
            text = []
            for e in self.GetPageElements(addBoxContents=1, addColumnContents=1):
                text.append(e.GetTexts())
            #!fulltext storage root
            #self.dbEntry.WriteFulltext(self.FormatFulltext(text))
        else:
            self.GetPage().UpdateFulltext()
            
    def FormatFulltext(self, textlist):
        # formats the raw text better display
        text = u"\n".join(textlist)
        text = text.replace(u"\n\n\n", u"\n")
        text = text.replace(u"\n\n", u"\n")
        return text

    def GetTexts(self):
        # loop all fulltext fields and make one string
        text = []
        root = self.root()
        for f in self.app.GetAllMetaFlds(ignoreSystem=False):
            if f.get("fulltext") != True:
                continue
            #print inUnit.GetID(),f
            if f["datatype"]=="unit":
                id = self.meta.get(f["id"])
                t = root.LookupObjTitle(id)
                if t:
                    text.append(t)
            elif f["datatype"]=="unitlist":
                ids = ConvertToNumberList(self.meta.get(f["id"]))
                for id in ids:
                    t = root.LookupObjTitle(id)
                    if t:
                        text.append(t)
            else:
                t = self.meta.get(f["id"],u"")
                if t:
                    text.append(ConvertHTMLToText(t, url="", removeReST=True))

        # data
        for f in self.configuration.data:
            if f.get("fulltext") != True:
                continue
            if f["datatype"]=="unit":
                id = self.data.get(f["id"])
                t = root.LookupObjTitle(id)
                if t:
                    text.append(t)
            elif f["datatype"]=="unitlist":
                ids = ConvertToNumberList(self.data.get(f["id"]))
                for id in ids:
                    t = root.LookupObjTitle(id)
                    if t:
                        text.append(t)
            else:
                t = self.data.get(f["id"],u"")
                if t:
                    text.append(ConvertHTMLToText(t, url="", removeReST=True))
        return u"\n\n".join(text)


    def GetFulltext(self):
        """
        Get current fulltext value
        """
        if not self.app.configuration.fulltextIndex:
            return u""
        return self.dbEntry.GetFulltext()

    
    def DeleteFulltext(self):
        """
        Delete fulltext
        """
        if not self.app.configuration.fulltextIndex:
            return
        self.dbEntry.DeleteFulltext()


class RewriteFulltext(Tool):

    def _Run(self, **values):

        try:
            localizer = get_localizer(get_current_request())
        except:
            localizer = FakeLocalizer()

        app = self.app
        root = app.root()
        datapool = app.db
        conn = datapool.connection
        c = conn.cursor()

        # delete
        sql = u"delete from pool_fulltext"
        c.execute(sql)
        c.close()
        conn.commit()
        self.stream.write(localizer.translate(_(u"Deleted previous fulltext index.<br/>")))
        
        pages = root.Select(parameter={"pool_stag":10,"pool_state":1})
        cnt = len(pages)
        err = 0
        for page in pages:
            page = page[0]
            obj = root.LookupObj(page)
            if not obj:
                err += 1
                self.stream.write(localizer.translate(_(u"Error: Unable to open page (${id}).<br/>", mapping={"id":page})))
            else:
                try:
                    obj.UpdateFulltext()
                except Exception, e:
                    err += 1
                    self.stream.write(localizer.translate(_(u"Error: Unable to update page (${id}).<br/>", mapping={"id":page})))
                    self.stream.write(unicode(e))
                    self.stream.write(u"<br/><br/>")
        conn.commit()
        self.stream.write(localizer.translate(_(u"Updated fulltext index. Finished.<br/>")))
        self.stream.write(localizer.translate(_(u"${cnt} pages ok. ${err} failed.<br/>", mapping={"cnt":cnt,"err":err})))
        return 1



def SetupFulltext(app, pyramidConfig):
    # get all objects and add extension
    extension = "nive.components.extensions.fulltextpage.PageFulltext"
    def add(confs):
        for c in confs:
            e = c.extensions
            if e == None:
                e = []
            elif extension in e:
                continue
            if isinstance(e, tuple):
                e = list(e)
            e.append(extension)
            c.unlock()
            c.extensions = tuple(e)
            c.lock()
    
    add(app.GetAllRootConfs())
    add(app.GetAllObjectConfs())


toolconf = ToolConf(
    id = "updatefulltext",
    context = "nive.components.extensions.fulltextpage.RewriteFulltext",
    name = _(u"Rewrite fulltext index"),
    description = _("Delete and rewrite the fulltext index."),
    apply = (IApplication,),
    data = [],
    mimetype = "text/html"
)

configuration = ModuleConf(
    id = "pagefulltext",
    name = u"Web page fulltext extension",
    context = PageFulltext,
    events = (Conf(event="startRegistration", callback=SetupFulltext),),
    modules = [toolconf]
)





# -*- coding: latin-1 -*-

import copy
from time import time
import unittest

from nive.definitions import DatabaseConf
from nive.utils.dataPool2.connection import *
from nive.utils.dataPool2.sqlite3Pool import Sqlite3

from sqlite3 import OperationalError

from nive.tests.db_app import app_db

from nive.tests import __local

# configuration ---------------------------------------------------------------------------
conn = DatabaseConf(
    dbName = __local.ROOT+"nive.db"
)


class ConnectionTest(unittest.TestCase):
    """
    """
    def test_conn(self):
        self.assert_(Connection(conn))
        
    def test_conntl(self):
        self.assert_(ConnectionThreadLocal(conn))
    
    def test_connreq(self):
        self.assert_(ConnectionRequest(conn))







import time
import unittest
import logging
from smtplib import SMTPServerDisconnected

from nive.definitions import *
from nive.components.tools.sendMail import *

from nive.tests import db_app


from nive.helper import FormatConfTestFailure



# -----------------------------------------------------------------

class SendMailTest1(unittest.TestCase):


    def test_conf1(self):
        r=configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_tool(self):
        sendMail(configuration,None)
        
        
class SendMailTest2_db(unittest.TestCase):

    def setUp(self):
        self.app = db_app.app_db()
        self.app.Include(configuration)
        logging.basicConfig()
    
    def tearDown(self):
        self.app.Close()


    def test_tool(self):
        t = self.app.GetTool("sendMail")
        self.assert_(t)

    
    def test_toolrun1(self):
        t = self.app.GetTool("sendMail")
        self.assert_(t)
        r,v = t()
        self.assertFalse(r)


    def test_toolrun2(self):
        t = self.app.GetTool("sendMail")
        self.assert_(t)
        try:
            r,v = t(recvmails=[("test@aaaaaaaa.com", "No name")], title="Testmail", body="body mail")
        except ConfigurationError:
            pass


    def test_toolrundebug(self):
        t = self.app.GetTool("sendMail")
        self.assert_(t)
        try:
            r,v = t(debug=1, recvmails=[("test@aaaaaaaa.com", "No name")], title="Testmail", body="body mail")
        except ConfigurationError:
            pass


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import unittest

from nive.utils import language_data
from nive.utils import language


class Langugage_data(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    
    def test_defs(self):
        for lang in language_data.languages:
            conf = language_data.GetConf(lang)
            self.assert_(conf.get("code"), lang)
            self.assert_(conf.get("code2"), lang)
            self.assert_(conf.get("name"), lang)
        conf = language_data.GetConf("xxx")
        self.assert_(conf.get("code")==u"")
        
    def test_codes(self):
        for conf in language_data.GetLanguages():
            self.assert_(conf.get("id"), conf)
            self.assert_(conf.get("name"), conf)
            
            
    def test_get(self):
        self.assert_(language_data.GetConf(u"ger").get(u"code2")==u"de")
        self.assert_(language_data.GetConf(u"de").get(u"code")==u"ger")
                        
        


class Langugage(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    
    def test_lang(self):
        lext = language.LanguageExtension()
        for c in lext.Codelist():
            self.assert_(c.get("id"))
            self.assert_(c.get("name"))
            
    def test_conf(self):
        lext = language.LanguageExtension()
        for l in language_data.languages:
            conf = lext.GetConf(l)
            self.assert_(conf.get("code"), l)
            self.assert_(conf.get("code2"), l)
            self.assert_(conf.get("name"), l)
            

    def test_name(self):
        lext = language.LanguageExtension()
        for l in language_data.languages:
            name = lext.GetName(l)
            self.assert_(name, l)
            
            
            
            
            




import unittest

from nive.definitions import *

from nive.components.extensions.persistence import *

from nive.tests import db_app

        

class Persistence(unittest.TestCase):
    
    def setUp(self):
        self.app = db_app.app_nodb()
        pass
    
    def tearDown(self):
        pass
    
    
    def test_1(self):
        p = PersistentConf(self.app, self.app.configuration)
        
        
    def test_2(self):
        LoadStoredConfValues(self.app, None)
        
        
        
class tdbPersistence(unittest.TestCase):
    
    def setUp(self):
        self.app = db_app.app_db(["nive.components.extensions.persistence.dbPersistenceConfiguration"])
        pass
    
    def tearDown(self):
        pass
    
    def test_conf1(self):
        r=dbPersistenceConfiguration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")
        
        
    def test_2(self):
        storage = self.app.Factory(IModuleConf, "persistence")
        self.assert_(storage)
        LoadStoredConfValues(self.app, None)


#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Spacer
------
Styling element to add line breaks and vertical space between elements. 
"""

from nive.i18n import _
from nive.definitions import StagPageElement, ObjectConf, FieldConf
from nive.components.objects.base import PageElementBase


class spacer(PageElementBase):
    pass
    


# type definition ------------------------------------------------------------------
#@nive_module
configuration = ObjectConf(
    id = "spacer",
    name = _(u"Spacer"),
    dbparam = "spacers",
    context = "nive.cms.spacer.spacer",
    template = "spacer.pt",
    selectTag = StagPageElement,
    icon = "nive.cms.cmsview:static/images/types/spacer.png",
    description = _(u"Styling element to add line breaks and vertical space between elements.")
)

css =[  {'id': u'bo', 'name': _(u'Border')},
        {'id': u'h0', 'name': _(u'Invisible')},
        {'id': u'h1', 'name': _(u'1 line')},
        {'id': u'h2', 'name': _(u'2 lines')},
        {'id': u'h3', 'name': _(u'3 lines')},
        {'id': u'h4', 'name': _(u'4 lines')},
        ]

configuration.data = [
    FieldConf(id="cssClass", datatype="list", size=5, default=u"", listItems=css, name=_(u"Styling"), description=u"")
]

fields =["cssClass", "pool_groups"]
configuration.forms = {"create": {"fields":fields}, "edit": {"fields":fields}}

configuration.views = []





import unittest

from nive.userdb.tests import db_app

from nive.definitions import Conf, AppConf
from nive.components.extensions.sessionuser import *
from nive.portal import Portal
from nive.userdb.app import UserDB


class testobj(object):
    def Listen(self, name, fnc):
        pass

class Conftest(unittest.TestCase):
    
    def test_conf1(self):
        r=configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")


class CacheTest(unittest.TestCase):
    
    def setUp(self):
        self.cache = SessionUserCache(1234)
    
    def tearDown(self):
        pass
    
    def test_caching(self):
        user1 = SessionUser("user1", 1, Conf(), Conf())
        user2 = SessionUser("user2", 2, Conf(), Conf())
        user3 = SessionUser("user3", 3, Conf(), Conf())
        
        self.assertFalse(self.cache.Get("user1"))
        self.assertFalse(self.cache.GetAll())

        self.cache.Add(user1, "user1")
        self.assert_(self.cache.Get("user1"))
        self.assert_(len(self.cache.GetAll())==1)

        self.cache.Add(user1, "user1")
        self.assert_(self.cache.Get("user1"))
        self.assert_(len(self.cache.GetAll())==1)

        self.cache.Add(user2, "user2")
        self.assert_(self.cache.Get("user1"))
        self.assert_(self.cache.Get("user2"))
        self.assert_(len(self.cache.GetAll())==2)

        self.cache.Add(user3, "user3")
        self.assert_(self.cache.Get("user1"))
        self.assert_(self.cache.Get("user2"))
        self.assert_(self.cache.Get("user3"))
        self.assert_(len(self.cache.GetAll())==3)

        self.cache.Invalidate("user1")
        self.assertFalse(self.cache.Get("user1"))
        self.assert_(self.cache.Get("user2"))
        self.assert_(self.cache.Get("user3"))
        self.assert_(len(self.cache.GetAll())==2)

        self.cache.Purge()
        self.assertFalse(self.cache.Get("user1"))
        self.assert_(self.cache.Get("user2"))
        self.assert_(self.cache.Get("user3"))
        self.assert_(len(self.cache.GetAll())==2)

        self.cache.expires = 0
        self.cache.Purge()
        self.assertFalse(self.cache.Get("user1"))
        self.assertFalse(self.cache.Get("user2"))
        self.assertFalse(self.cache.Get("user3"))
        self.assert_(len(self.cache.GetAll())==0)

class ListenerTest(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def test_root(self):
        r = RootListener()
        r.app = testobj()
        r.app.usercache = SessionUserCache()
        user = r.LookupCache(ident="user1", activeOnly=None)
        self.assertFalse(user)
        r.app.usercache.Add(SessionUser("user1", 1, Conf(), Conf()), "user1")
        self.assertRaises(UserFound, r.LookupCache, ident="user1", activeOnly=None)

        u = UserListener()
        u.app = testobj()
        u.app.usercache = SessionUserCache()
        u.identity = "user1"
        u.data = Conf()
        u.id = 1
        sessionuser = r.SessionUserFactory("user1", u)
        self.assert_(sessionuser)
        r.AddToCache(sessionuser)
        
    def test_user(self):
        u = UserListener()
        u.app = testobj()
        u.app.usercache = SessionUserCache()
        u.identity = "user1"
        u.data = Conf()
        u.id = 1
        u.InvalidateCache()
        
        
class SessionuserTest(unittest.TestCase):
    
    def setUp(self):
        values = {"name":"user1", 
                  "email": "user@nive.co", 
                  "surname": "The", 
                  "lastname": "User", 
                  "groups": ("here", "there"), 
                  "lastlogin": time.time()}
        values2 = {"id": 1, "pool_state": 1}
        self.user = SessionUser("user1", 1, Conf(**values), Conf(**values2))
        pass
    
    def tearDown(self):
        pass
    
    
    def test_iface(self):
        i = ISessionUser
        
    def test_user(self):
        self.assert_(self.user.lastlogin)
        self.assert_(self.user.currentlogin)
        self.assert_(self.user.data)
        self.assert_(self.user.data.name)
        self.assert_(self.user.data.email)
        self.assert_(self.user.data.groups)
        self.assert_(self.user.meta.id==1)
        self.assert_(self.user.meta.pool_state==1)
    
    def test_groups(self):
        grps = self.user.GetGroups()
        self.assert_(self.user.data.groups==grps)
        
    def test_ingroups(self):
        self.assert_(self.user.InGroups("here"))
        self.assert_(self.user.InGroups(["there", "ohno"]))
        self.assertFalse(self.user.InGroups(["ahaha", "ohno"]))
        
        
class AppTest(unittest.TestCase):
    
    def setUp(self):
        pass
    def tearDown(self):
        pass

    def test_app(self):
        
        appconf = AppConf("nive.userdb.app")
        #appconf.modules.append("nive.components.extensions.sessionuser")
        
        app = UserDB(appconf)
        app.dbConfiguration=db_app.dbconf
        p = Portal()
        p.Register(app)
        app.Startup(None)
        
        self.assert_(app.usercache)
        
        self.assert_(app.root())



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Column
------
A column is a container to group elements on a page. Columns are not added like
normal page elements but have to be defined manually in the main template. A column can
be inherited through a hierarchy of pages.
"""

from nive.i18n import _
from nive.definitions import StagPage, StagPageElement, ObjectConf, FieldConf, IColumn, implements
from nive.components.objects.base import PageElementContainerBase


class column(PageElementContainerBase):
    implements(IColumn)
    
    @property
    def page(self):
        return self.parent

    def IsLocal(self, page):
        #
        return self.meta.pool_unitref == page.id
    
    def GetName(self):
        #
        return self.meta["title"]
    
    def IsContainer(self):
        #
        return True

    def IsPage(self):
        #
        return False

    def GetPage(self):
        # returns the current page
        return self.GetParent()

    def GetPages(self):
        # columns have no subpages
        return []

    def GetElementContainer(self):
        # returns the current element container
        return self 

    def GetContainer(self):
        # returns the current container
        return self.GetParent()

    def GetColumn(self, name):
        # 
        if name == self.meta["title"]:
            return self
        return self.GetPage().GetColumn(name)


# type definition ------------------------------------------------------------------
#@nive_module
configuration =  ObjectConf(
    id = "column",
    name = _(u"Column"),
    dbparam = "columnbox",
    context = "nive.cms.column.column",
    template = "column.pt",
    hidden = True,
    selectTag = StagPageElement,
    container = True,
    icon = "nive.cms.cmsview:static/images/types/column.png",
    description = _(u"A column is a container to group elements on a page. Columns are not added like"
                    u"normal page elements but have to be defined manually in the main template. A column can"
                    u"be inherited through a hierarchy of pages.")
)
configuration.data = [
    FieldConf(id="showsub", datatype="bool", size=2, default=1, name=_(u"Show on subpages"), description=_(u"If checked the column will be displayed on sub pages until overwritten."))
]

configuration.forms = {
        "create": {"fields": [FieldConf(id="title", datatype="string", size=10, default=u"", hidden=1, required=1, name=_(u"Column type")),
                              "showsub", "pool_groups"]},
        "edit":   {"fields": ["showsub", "pool_groups"]}
}

configuration.views = []


    
    



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Root
----
The *root* is the home page of the website. All contained pages and elements are stored 
in the database. The root itself does not store anything in the database.

Also this object provides search functions and sql query wrappers.
"""

from nive.i18n import _
from nive.components.objects.base import PageRootBase
from nive.definitions import IWebsiteRoot, implements
from nive.definitions import RootConf


class root(PageRootBase):

    implements(IWebsiteRoot)
    extension = u"html"
    
    def Init(self):
        self.queryRestraints = {"pool_state": 1}, {}
    
    


# Root definition ------------------------------------------------------------------
#@nive_module
configuration = RootConf(
    id = "root",
    context = "nive.cms.root.root",
    template = "root.pt",
    default = True,
    subtypes = "*",
    extensions = ("nive.components.extensions.persistentRoot.Persistent",),
    name = _(u"Home"),
    description = _(u"The root is the home page of the website. All contained pages and elements are stored in the database.")
)



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
The *portal* manages applications and is the pyramid_ or wsgi_ root. There is only one
portal in each instance.

The portal handles the cms applications and global components like the user database. 
Applications have to be registered by calling ``Register(application)`` on initialisation.

- Routes url names to applications
- Calls ``Startup()`` for each registered application on server startup
- Provides connection start and finish events
- Collects all registered security groups from applications 

Allows by default all permissions for admins ::

    __acl__ = (Allow, "group:admin", ALL_PERMISSIONS)

Also redirects for login, forbidden, logout and default url are configured here ::

        portalDefaultUrl = "/website/"
        loginUrl = "/userdb/udb/login"
        loginSuccessUrl = "/website/"
        forbiddenUrl = "/userdb/udb/login"
        logoutUrl = "/userdb/udb/logout"
        logoutSuccessUrl = "/website/"
        accountUrl = "/userdb/udb/update"

Interface: IPortal
"""

import copy
import time
import logging
import gc

from pyramid.events import NewRequest

from nive.i18n import _
from nive.definitions import IPortal, PortalConf, implements, Conf
from nive.definitions import ConfigurationError
from nive.helper import ResolveConfiguration, ResolveName
from nive.security import User, authenticated_userid, Allow, ALL_PERMISSIONS
from nive.events import Events
from nive.utils.utils import SortConfigurationList



class Portal(Events, object):
    """ """
    implements(IPortal)

    __name__ = u""
    __parent__ = None
    
    def __init__(self, configuration = None):
        """
        Events:
        - init(configuration)
        """
        self.components = []
        self.groups = [Conf(id=u"authenticated", name=_(u"Authenticated"), visible=True)]
        self.__acl__ = [(Allow, "group:admin", ALL_PERMISSIONS)]
        
        self.configuration = configuration or PortalConf()
        
        self.Signal("init", configuration=self.configuration)

    def __del__(self):
        self.Close()
        
    def __getitem__(self, name):
        """
        Called by traversal machinery
        
        event: getitem(obj) called with the traversed object
        """
        try:
            obj = getattr(self, name)
            self.Signal("getitem", obj=obj)
            return obj
        except AttributeError:
            raise KeyError, name
        
    def Register(self, comp, name=None):
        """
        Register an application or component. This is usually done in the pyramid
        app file. The registered component is automatically loaded and set up to work
        with url traversal.
        
        *comp* can be one of the following cases

        - AppConf object
        - AppConf string as python dotted name
        - python object. Requires *name* parameter or *comp.id* attribute
        
        *name* is used as the url path name to lookup the component.

        """
        log = logging.getLogger("portal")
        iface, conf = ResolveConfiguration(comp)
        if not conf:
            if not name or isinstance(comp, basestring):
                raise ConfigurationError, "Portal registration failure. No name given (%s)" % (str(comp))
        elif isinstance(comp, basestring):
            c = ResolveName(conf.context)
            comp = c(conf)
        elif iface and iface.providedBy(comp):
            c = ResolveName(conf.context)
            comp = c(conf)
        try:
            if not name:
                name = conf.id
        except:
            pass
        if not name:
            raise ConfigurationError, "Portal registration failure. No name given (%s)" % (str(comp))

        log.debug("Portal.Register: %s %s", name, repr(conf))
        self.__dict__[name] = comp
        comp.__parent__ = self
        comp.__name__ = name
        self.components.append(name)
        #self.RegisterGroups(comp)


    def Startup(self, pyramidConfig, debug=False):
        """
        *Startup* is called once by the *main* function of the pyramid wsgi app on 
        server startup. All configuration, registration and setup is handled during
        the startup call. Calls *Startup()* for each registered component.
        
        *pyramidConfig* is the pyramid registration configuration object for views and other 
        system components. nive ViewConf_ and ViewModuelConf_ are automatically with
        pyramid. 
        
        *debug* signals whether running in debug mode or not.
        """
        log = logging.getLogger("portal")
        log.debug("Portal.Startup with debug=%s", str(debug))
        if pyramidConfig:
            self.SetupPortalViews(pyramidConfig)
            #pyramidConfig.add_subscriber(self.StartConnection, iface=NewRequest)
        for c in self.components:
            component = getattr(self, c)
            if hasattr(component, "Startup"):
                component.Startup(pyramidConfig, debug=debug)


    def GetApps(self, interface=None):
        """
        Returns registered components and apps as list.
        """
        if isinstance(interface, basestring):
            interface = ResolveName(interface)
        apps=[]
        for name in self.components:
            a = getattr(self, name)
            if interface:
                if not interface.providedBy(a):
                    continue
            apps.append(a)
        return apps


    def StartConnection(self, event):
        """
        Called by pyramid for each new connection with event as parameter. The
        current request stored as *event.request*. Stores the authenticated user 
        *event.request.environ["REMOTE_USER"]*.
        
        Event:
        - start(event)
        """
        uid = authenticated_userid(event.request)
        event.request.environ["REMOTE_USER"] = uid
        event.request.environ["START_TIME"] = time.time()
        self.Signal("start", event=event)
        #event.request.add_finished_callback(self.FinishConnection)


    def FinishConnection(self, request):
        """
        Called by pyramid on termination for each connection with request as parameter.

        Event:
        - finish(request)
        """
        self.Signal("finish", request)
        

    def GetGroups(self, sort=u"id", visibleOnly=False):
        """
        returns all groups registered by components as list
        """
        if visibleOnly:
            #opt
            g = []
            for a in self.groups:
                if not a.get("hidden"):
                    g.append(a)
        else:
            g = self.groups
        if not sort:
            return g
        l = copy.deepcopy(g)
        return SortConfigurationList(l, sort)


    @property
    def portal(self):
        return self
    

    def RegisterGroups(self, component):
        """
        Collects groups from the component
        """
        try:
            gr = component.configuration.groups
            for g in gr:
                add = 1
                for g2 in self.groups:
                    if g["id"] == g2["id"]:
                        add = 0
                        break
                if add:
                    self.groups.append(g)
        except:
            pass
                

    def SetupPortalViews(self, config):
        # redirects
        config.add_view(error_view, context=HTTPError)
        config.add_view(forbidden_view, context=Forbidden)
        config.add_view(portal_view, name="", context="nive.portal.Portal")
        config.add_view(robots_view, name="robots.txt", context="nive.portal.Portal")
        config.add_view(sitemap_view, name="sitemap.xml", context="nive.portal.Portal")
        config.add_view(logout_view, name="logout", context="nive.portal.Portal")
        config.add_view(login_view,  name="login", context="nive.portal.Portal")
        config.add_view(account_view,name="account", context="nive.portal.Portal")
        #config.add_view(favicon_view, name=u"favicon.txt", context=u"nive.portal.Portal", view=PortalViews)
    
        # translations
        config.add_translation_dirs('nive:locale/')
        
        config.commit()

    
    def Close(self):
        for name in self.components:
            a = getattr(self, name)
            a.Close()
            setattr(self, name, None)
            

    # bw 0.9.4
    def GetComponents(self):
        return self.GetApps()





from views import Redirect
from pyramid.exceptions import Forbidden
from pyramid.response import Response
from pyramid.httpexceptions import HTTPError, HTTPNotFound, HTTPServerError


def forbidden_view(request):
    # login form
    url = request.referrer
    portal = request.context.app.portal
    return Redirect(portal.configuration.forbiddenUrl+"?redirect="+request.url, request, messages=[u"Please log in"])
       
def login_view(context, request):
    # login form
    portal = context
    return Redirect(portal.configuration.loginUrl+"?redirect="+request.GET.get("redirect",""), request)
    
def logout_view(context, request):
    # logout to login form
    portal = context
    return Redirect(portal.configuration.logoutUrl+"?redirect="+request.GET.get("redirect",""), request)
    
def portal_view(context, request):
    # website root / domain root redirect
    portal = context
    return Redirect(portal.configuration.portalDefaultUrl, request)
    
def account_view(context, request):
    # account page
    portal = context
    return Redirect(portal.configuration.accountUrl, request)
    
def robots_view(context, request):
    portal = request.context
    # website root / domain root redirect
    r = Response(content_type="text/plain", conditional_response=False)
    r.unicode_body = portal.configuration.robots
    return r

def sitemap_view(context, request):
    portal = request.context
    # portal google sitemap link
    r = Response(content_type="text/xml", conditional_response=False)
    r.unicode_body = portal.configuration.sitemap
    return r

def error_view(context, request):
    #context.body = str(context)
    return context





    



import unittest

def invalid_exc(func, *arg, **kw):
    from nive.components.reform.schema import Invalid
    try:
        func(*arg, **kw)
    except Invalid, e:
        return e
    else:
        raise AssertionError('Invalid not raised') # pragma: no cover

class TestInvalid(unittest.TestCase):
    def _makeOne(self, node, msg=None, val=None):
        from nive.components.reform.schema import Invalid
        exc = Invalid(node, msg, val)
        return exc

    def test_ctor(self):
        exc = self._makeOne(None, 'msg', 'val')
        self.assertEqual(exc.node, None)
        self.assertEqual(exc.msg, 'msg')
        self.assertEqual(exc.value, 'val')
        self.assertEqual(exc.children, [])

    def test_add(self):
        exc = self._makeOne(None, 'msg')
        other = Dummy()
        exc.add(other)
        self.failIf(hasattr(other, 'positional'))
        self.assertEqual(exc.children, [other])

    def test_add_positional(self):
        from nive.components.reform.schema import Positional
        p = Positional()
        node = DummySchemaNode(p)
        exc = self._makeOne(node, 'msg')
        other = Dummy()
        exc.add(other)
        self.assertEqual(other.positional, True)
        self.assertEqual(exc.children, [other])

    def test__keyname_no_parent(self):
        node = DummySchemaNode(None, name='name')
        exc = self._makeOne(None, '')
        exc.node = node
        self.assertEqual(exc._keyname(), 'name')

    def test__keyname_positional(self):
        exc = self._makeOne(None, '')
        exc.positional = True
        exc.pos = 2
        self.assertEqual(exc._keyname(), '2')

    def test__keyname_nonpositional_parent(self):
        parent = Dummy()
        parent.node = DummySchemaNode(None)
        exc = self._makeOne(None, 'me')
        exc.parent = parent
        exc.pos = 2
        exc.node = DummySchemaNode(None, name='name')
        self.assertEqual(exc._keyname(), 'name')

    def test_paths(self):
        exc1 = self._makeOne(None, 'exc1')
        exc2 = self._makeOne(None, 'exc2')
        exc3 = self._makeOne(None, 'exc3')
        exc4 = self._makeOne(None, 'exc4')
        exc1.add(exc2)
        exc2.add(exc3)
        exc1.add(exc4)
        paths = list(exc1.paths())
        self.assertEqual(paths, [(exc1, exc2, exc3), (exc1, exc4)])

    def test_asdict(self):
        from nive.components.reform.schema import Positional
        node1 = DummySchemaNode(None, 'node1')
        node2 = DummySchemaNode(Positional(), 'node2')
        node3 = DummySchemaNode(Positional(), 'node3')
        node4 = DummySchemaNode(Positional(), 'node4')
        exc1 = self._makeOne(node1, 'exc1')
        exc1.pos = 1
        exc2 = self._makeOne(node2, 'exc2')
        exc3 = self._makeOne(node3, 'exc3')
        exc4 = self._makeOne(node4, 'exc4')
        exc1.add(exc2, 2)
        exc2.add(exc3, 3)
        exc1.add(exc4, 4)
        d = exc1.asdict()
        self.assertEqual(d, {'node1.node2.3': 'exc1; exc2; exc3',
                             'node1.node4': 'exc1; exc4'})

    def test___str__(self):
        from nive.components.reform.schema import Positional
        node1 = DummySchemaNode(None, 'node1')
        node2 = DummySchemaNode(Positional(), 'node2')
        node3 = DummySchemaNode(Positional(), 'node3')
        node4 = DummySchemaNode(Positional(), 'node4')
        exc1 = self._makeOne(node1, 'exc1')
        exc1.pos = 1
        exc2 = self._makeOne(node2, 'exc2')
        exc3 = self._makeOne(node3, 'exc3')
        exc4 = self._makeOne(node4, 'exc4')
        exc1.add(exc2, 2)
        exc2.add(exc3, 3)
        exc1.add(exc4, 4)
        result = str(exc1)
        self.assertEqual(
            result,
            "{'node1.node2.3': 'exc1; exc2; exc3', 'node1.node4': 'exc1; exc4'}"
            )

    def test___setitem__fails(self):
        node = DummySchemaNode(None)
        exc = self._makeOne(node, 'msg')
        self.assertRaises(KeyError, exc.__setitem__, 'notfound', 'msg')

    def test___setitem__succeeds(self):
        node = DummySchemaNode(None)
        child = DummySchemaNode(None)
        child.name = 'found'
        node.children = [child]
        exc = self._makeOne(node, 'msg')
        exc['found'] = 'msg2'
        self.assertEqual(len(exc.children), 1)
        childexc = exc.children[0]
        self.assertEqual(childexc.pos, 0)
        self.assertEqual(childexc.node.name, 'found')

    def test_messages_msg_iterable(self):
        node = DummySchemaNode(None)
        exc = self._makeOne(node, [123, 456])
        self.assertEqual(exc.messages(), [123, 456])

    def test_messages_msg_not_iterable(self):
        node = DummySchemaNode(None)
        exc = self._makeOne(node, 'msg')
        self.assertEqual(exc.messages(), ['msg'])

class TestAll(unittest.TestCase):
    def _makeOne(self, validators):
        from nive.components.reform.schema import All
        return All(*validators)

    def test_success(self):
        validator1 = DummyValidator()
        validator2 = DummyValidator()
        validator = self._makeOne([validator1, validator2])
        self.assertEqual(validator(None, None), None)

    def test_failure(self):
        validator1 = DummyValidator('msg1')
        validator2 = DummyValidator('msg2')
        validator = self._makeOne([validator1, validator2])
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, ['msg1', 'msg2'])

class TestFunction(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from nive.components.reform.schema import Function
        return Function(*arg, **kw)

    def test_success_function_returns_True(self):
        validator = self._makeOne(lambda x: True)
        self.assertEqual(validator(None, None), None)

    def test_fail_function_returns_empty_string(self):
        validator = self._makeOne(lambda x: '')
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, 'Invalid value')

    def test_fail_function_returns_False(self):
        validator = self._makeOne(lambda x: False)
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, 'Invalid value')

    def test_fail_function_returns_string(self):
        validator = self._makeOne(lambda x: 'fail')
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg, 'fail')

    def test_propagation(self):
        validator = self._makeOne(lambda x: 'a' in x, 'msg')
        self.assertRaises(TypeError, validator, None, None)

class TestRange(unittest.TestCase):
    def _makeOne(self, **kw):
        from nive.components.reform.schema import Range
        return Range(**kw)

    def test_success_no_bounds(self):
        validator = self._makeOne()
        self.assertEqual(validator(None, 1), None)

    def test_success_upper_bound_only(self):
        validator = self._makeOne(max=1)
        self.assertEqual(validator(None, -1), None)

    def test_success_minimum_bound_only(self):
        validator = self._makeOne(min=0)
        self.assertEqual(validator(None, 1), None)

    def test_success_min_and_max(self):
        validator = self._makeOne(min=1, max=1)
        self.assertEqual(validator(None, 1), None)

    def test_min_failure(self):
        validator = self._makeOne(min=1)
        e = invalid_exc(validator, None, 0)
        self.assertEqual(e.msg.interpolate(), '0 is less than minimum value 1')

    def test_min_failure_msg_override(self):
        validator = self._makeOne(min=1, min_err='wrong')
        e = invalid_exc(validator, None, 0)
        self.assertEqual(e.msg, 'wrong')

    def test_max_failure(self):
        validator = self._makeOne(max=1)
        e = invalid_exc(validator, None, 2)
        self.assertEqual(e.msg.interpolate(),
                         '2 is greater than maximum value 1')

    def test_max_failure_msg_override(self):
        validator = self._makeOne(max=1, max_err='wrong')
        e = invalid_exc(validator, None, 2)
        self.assertEqual(e.msg, 'wrong')

class TestRegex(unittest.TestCase):
    def _makeOne(self, pattern):
        from nive.components.reform.schema import Regex
        return Regex(pattern)

    def test_valid_regex(self):
        self.assertEqual(self._makeOne('a')(None, 'a'), None)
        self.assertEqual(self._makeOne('[0-9]+')(None, '1111'), None)
        self.assertEqual(self._makeOne('')(None, ''), None)
        self.assertEqual(self._makeOne('.*')(None, ''), None)

    def test_invalid_regexs(self):
        from nive.components.reform.schema import Invalid
        self.assertRaises(Invalid, self._makeOne('[0-9]+'), None, 'a')
        self.assertRaises(Invalid, self._makeOne('a{2,4}'), None, 'ba')

    def test_regex_not_string(self):
        from nive.components.reform.schema import Invalid
        import re
        regex = re.compile('[0-9]+')
        self.assertEqual(self._makeOne(regex)(None, '01'), None)
        self.assertRaises(Invalid, self._makeOne(regex), None, 't')


class TestEmail(unittest.TestCase):
    def _makeOne(self):
        from nive.components.reform.schema import Email
        return Email()

    def test_valid_emails(self):
        validator = self._makeOne()
        self.assertEqual(validator(None, 'me@here.com'), None)
        self.assertEqual(validator(None, 'me1@here1.com'), None)
        self.assertEqual(validator(None, 'name@here1.us'), None)
        self.assertEqual(validator(None, 'name@here1.info'), None)
        self.assertEqual(validator(None, 'foo@bar.baz.biz'), None)

    def test_empty_email(self):
        validator = self._makeOne()
        e = invalid_exc(validator, None, '')
        self.assertEqual(e.msg, 'Invalid email address')

    def test_invalid_emails(self):
        validator = self._makeOne()
        from nive.components.reform.schema import Invalid
        self.assertRaises(Invalid, validator, None, 'me@here.')
        self.assertRaises(Invalid, validator, None, 'name@here.comcom')
        self.assertRaises(Invalid, validator, None, '@here.us')
        self.assertRaises(Invalid, validator, None, '(name)@here.info')

class TestLength(unittest.TestCase):
    def _makeOne(self, min=None, max=None):
        from nive.components.reform.schema import Length
        return Length(min=min, max=max)

    def test_success_no_bounds(self):
        validator = self._makeOne()
        self.assertEqual(validator(None, ''), None)

    def test_success_upper_bound_only(self):
        validator = self._makeOne(max=1)
        self.assertEqual(validator(None, 'a'), None)

    def test_success_minimum_bound_only(self):
        validator = self._makeOne(min=0)
        self.assertEqual(validator(None, ''), None)

    def test_success_min_and_max(self):
        validator = self._makeOne(min=1, max=1)
        self.assertEqual(validator(None, 'a'), None)

    def test_min_failure(self):
        validator = self._makeOne(min=1)
        e = invalid_exc(validator, None, '')
        self.assertEqual(e.msg.interpolate(), 'Shorter than minimum length 1')

    def test_max_failure(self):
        validator = self._makeOne(max=1)
        e = invalid_exc(validator, None, 'ab')
        self.assertEqual(e.msg.interpolate(), 'Longer than maximum length 1')

class TestOneOf(unittest.TestCase):
    def _makeOne(self, values):
        from nive.components.reform.schema import OneOf
        return OneOf(values)

    def test_success(self):
        validator = self._makeOne([1])
        self.assertEqual(validator(None, 1), None)

    def test_failure(self):
        validator = self._makeOne([1, 2])
        e = invalid_exc(validator, None, None)
        self.assertEqual(e.msg.interpolate(), '"None" is not one of 1, 2')

class TestSchemaType(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from nive.components.reform.schema import SchemaType
        return SchemaType(*arg, **kw)

    def test_flatten(self):
        node = DummySchemaNode(None, name='node')
        typ = self._makeOne()
        result = typ.flatten(node, 'appstruct')
        self.assertEqual(result, {'node':'appstruct'})

    def test_flatten_listitem(self):
        node = DummySchemaNode(None, name='node')
        typ = self._makeOne()
        result = typ.flatten(node, 'appstruct', listitem=True)
        self.assertEqual(result, {'':'appstruct'})

    def test_unflatten(self):
        node = DummySchemaNode(None, name='node')
        typ = self._makeOne()
        result = typ.unflatten(node, ['node'], {'node': 'appstruct'})
        self.assertEqual(result, 'appstruct')

    def test_set_value(self):
        typ = self._makeOne()
        self.assertRaises(
            AssertionError, typ.set_value, None, None, None, None)

    def test_get_value(self):
        typ = self._makeOne()
        self.assertRaises(
            AssertionError, typ.get_value, None, None, None)

class TestMapping(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from nive.components.reform.schema import Mapping
        return Mapping(*arg, **kw)

    def test_ctor_bad_unknown(self):
        self.assertRaises(ValueError, self._makeOne, 'badarg')

    def test_ctor_good_unknown(self):
        try:
            self._makeOne('ignore')
            self._makeOne('raise')
            self._makeOne('preserve')
        except ValueError, e: # pragma: no cover
            raise AssertionError(e)

    def test_deserialize_not_a_mapping(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, None)
        self.failUnless(
            e.msg.interpolate().startswith('"None" is not a mapping type'))

    def test_deserialize_null(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, reform.schema.null)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_no_subnodes(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, {})
        self.assertEqual(result, {})

    def test_deserialize_ok(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        result = typ.deserialize(node, {'a':1})
        self.assertEqual(result, {'a':1})

    def test_deserialize_unknown_raise(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne(unknown='raise')
        e = invalid_exc(typ.deserialize, node, {'a':1, 'b':2})
        self.assertEqual(e.msg.interpolate(),
                         "Unrecognized keys in mapping: \"{'b': 2}\"")


    def test_deserialize_unknown_preserve(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne(unknown='preserve')
        result = typ.deserialize(node, {'a':1, 'b':2})
        self.assertEqual(result, {'a':1, 'b':2})

    def test_deserialize_subnodes_raise(self):
        node = DummySchemaNode(None)
        node.children = [
            DummySchemaNode(None, name='a', exc='Wrong 2'),
            DummySchemaNode(None, name='b', exc='Wrong 2'),
            ]
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, {'a':1, 'b':2})
        self.assertEqual(e.msg, None)
        self.assertEqual(len(e.children), 2)

    def test_deserialize_subnode_missing_default(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        node.children = [
            DummySchemaNode(None, name='a'),
            DummySchemaNode(None, name='b', default='abc'),
            ]
        typ = self._makeOne()
        result = typ.deserialize(node, {'a':1})
        self.assertEqual(result, {'a':1, 'b':reform.schema.null})

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, {})

    def test_serialize_not_a_mapping(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, None)
        self.failUnless(
            e.msg.interpolate().startswith('"None" is not a mapping type'))

    def test_serialize_no_subnodes(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, {})
        self.assertEqual(result, {})

    def test_serialize_ok(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        result = typ.serialize(node, {'a':1})
        self.assertEqual(result, {'a':1})

    def test_serialize_with_unknown(self):
        node = DummySchemaNode(None)
        node.children = [
            DummySchemaNode(None, name='a'),
            ]
        typ = self._makeOne()
        result = typ.serialize(node, {'a':1, 'b':2})
        self.assertEqual(result, {'a':1})

    def test_serialize_value_is_null(self):
        node = DummySchemaNode(None)
        from nive.components.reform.schema import null
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        result = typ.serialize(node, null)
        self.assertEqual(result, {'a':null})

    def test_flatten(self):
        node = DummySchemaNode(None, name='node')
        int1 = DummyType()
        int2 = DummyType()
        node.children = [
            DummySchemaNode(int1, name='a'),
            DummySchemaNode(int2, name='b'),
            ]
        typ = self._makeOne()
        result = typ.flatten(node, {'a':1, 'b':2})
        self.assertEqual(result, {'node.appstruct': 2})

    def test_flatten_listitem(self):
        node = DummySchemaNode(None, name='node')
        int1 = DummyType()
        int2 = DummyType()
        node.children = [
            DummySchemaNode(int1, name='a'),
            DummySchemaNode(int2, name='b'),
            ]
        typ = self._makeOne()
        result = typ.flatten(node, {'a':1, 'b':2}, listitem=True)
        self.assertEqual(result, {'appstruct': 2})

    def test_unflatten(self):
        node = DummySchemaNode(None, name='node')
        int1 = DummyType()
        int2 = DummyType()
        node.children = [
            DummySchemaNode(int1, name='a'),
            DummySchemaNode(int2, name='b'),
            ]
        typ = self._makeOne()
        result = typ.unflatten(node,
            ['node', 'node.a', 'node.b'],
            {'node': {'a':1, 'b':2}, 'node.a':1, 'node.b':2})
        self.assertEqual(result, {'a': 1, 'b': 2})

    def test_unflatten_nested(self):
        node = DummySchemaNode(None, name='node')
        inttype = DummyType()
        one = DummySchemaNode(self._makeOne(), name='one')
        one.children = [
            DummySchemaNode(inttype, name='a'),
            DummySchemaNode(inttype, name='b'),
        ]
        two = DummySchemaNode(self._makeOne(), name='two')
        two.children = [
            DummySchemaNode(inttype, name='c'),
            DummySchemaNode(inttype, name='d'),
        ]
        node.children = [one, two]
        typ = self._makeOne()
        result = typ.unflatten(
            node, ['node', 'node.one', 'node.one.a', 'node.one.b',
                   'node.two', 'node.two.c', 'node.two.d'],
            {'node': {'one': {'a': 1, 'b': 2}, 'two': {'c': 3, 'd': 4}},
             'node.one': {'a': 1, 'b': 2},
             'node.two': {'c': 3, 'd': 4},
             'node.one.a': 1,
             'node.one.b': 2,
             'node.two.c': 3,
             'node.two.d': 4,})
        self.assertEqual(result, {
            'one': {'a': 1, 'b': 2}, 'two': {'c': 3, 'd': 4}})

    def test_set_value(self):
        typ = self._makeOne()
        node1 = DummySchemaNode(typ, name='node1')
        node2 = DummySchemaNode(typ, name='node2')
        node1.children = [node2]
        appstruct = {'node2': {'foo': 'foo', 'baz': 'baz'}}
        typ.set_value(node1, appstruct, 'node2.foo', 'bar')
        self.assertEqual(appstruct, {'node2': {'foo': 'bar', 'baz': 'baz'}})

    def test_get_value(self):
        typ = self._makeOne()
        node1 = DummySchemaNode(typ, name='node1')
        node2 = DummySchemaNode(typ, name='node2')
        node1.children = [node2]
        appstruct = {'node2': {'foo': 'bar', 'baz': 'baz'}}
        self.assertEqual(typ.get_value(node1, appstruct, 'node2'),
                         {'foo': 'bar', 'baz': 'baz'})
        self.assertEqual(typ.get_value(node1, appstruct, 'node2.foo'), 'bar')


class TestTuple(unittest.TestCase):
    def _makeOne(self):
        from nive.components.reform.schema import Tuple
        return Tuple()

    def test_deserialize_not_iterable(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, None)
        self.assertEqual(
            e.msg.interpolate(),
            '"None" is not iterable')
        self.assertEqual(e.node, node)

    def test_deserialize_no_subnodes(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, ())
        self.assertEqual(result, ())

    def test_deserialize_null(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, reform.schema.null)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_ok(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        result = typ.deserialize(node, ('a',))
        self.assertEqual(result, ('a',))

    def test_deserialize_toobig(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, ('a','b'))
        self.assertEqual(e.msg.interpolate(),
      "\"('a', 'b')\" has an incorrect number of elements (expected 1, was 2)")

    def test_deserialize_toosmall(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, ())
        self.assertEqual(e.msg.interpolate(),
           '"()" has an incorrect number of elements (expected 1, was 0)')

    def test_deserialize_subnodes_raise(self):
        node = DummySchemaNode(None)
        node.children = [
            DummySchemaNode(None, name='a', exc='Wrong 2'),
            DummySchemaNode(None, name='b', exc='Wrong 2'),
            ]
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, ('1', '2'))
        self.assertEqual(e.msg, None)
        self.assertEqual(len(e.children), 2)

    def test_serialize_null(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, reform.schema.null)
        self.assertEqual(result, reform.schema.null)

    def test_serialize_no_subnodes(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, ())
        self.assertEqual(result, ())

    def test_serialize_ok(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        result = typ.serialize(node, ('a',))
        self.assertEqual(result, ('a',))

    def test_serialize_toobig(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, ('a','b'))
        self.assertEqual(e.msg.interpolate(),
     "\"('a', 'b')\" has an incorrect number of elements (expected 1, was 2)")

    def test_serialize_toosmall(self):
        node = DummySchemaNode(None)
        node.children = [DummySchemaNode(None, name='a')]
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, ())
        self.assertEqual(e.msg.interpolate(),
           '"()" has an incorrect number of elements (expected 1, was 0)'
           )

    def test_serialize_subnodes_raise(self):
        node = DummySchemaNode(None)
        node.children = [
            DummySchemaNode(None, name='a', exc='Wrong 2'),
            DummySchemaNode(None, name='b', exc='Wrong 2'),
            ]
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, ('1', '2'))
        self.assertEqual(e.msg, None)
        self.assertEqual(len(e.children), 2)

    def test_flatten(self):
        node = DummySchemaNode(None, name='node')
        int1 = DummyType()
        int2 = DummyType()
        node.children = [
            DummySchemaNode(int1, name='a'),
            DummySchemaNode(int2, name='b'),
            ]
        typ = self._makeOne()
        result = typ.flatten(node, (1, 2))
        self.assertEqual(result, {'node.appstruct': 2})

    def test_flatten_listitem(self):
        node = DummySchemaNode(None, name='node')
        int1 = DummyType()
        int2 = DummyType()
        node.children = [
            DummySchemaNode(int1, name='a'),
            DummySchemaNode(int2, name='b'),
            ]
        typ = self._makeOne()
        result = typ.flatten(node, (1, 2), listitem=True)
        self.assertEqual(result, {'appstruct': 2})

    def test_unflatten(self):
        node = DummySchemaNode(None, name='node')
        int1 = DummyType()
        int2 = DummyType()
        node.children = [
            DummySchemaNode(int1, name='a'),
            DummySchemaNode(int2, name='b'),
            ]
        typ = self._makeOne()
        result = typ.unflatten(node, ['node', 'node.a', 'node.b'],
                               {'node': (1, 2), 'node.a': 1, 'node.b': 2})
        self.assertEqual(result, (1, 2))

    def test_set_value(self):
        typ = self._makeOne()
        node = DummySchemaNode(typ, name='node')
        node.children = [
            DummySchemaNode(typ, name='foo'),
            DummySchemaNode(typ, name='bar')
        ]
        node['foo'].children = [
            DummySchemaNode(None, name='a'),
            DummySchemaNode(None, name='b'),
        ]
        node['bar'].children = [
            DummySchemaNode(None, name='c'),
            DummySchemaNode(None, name='d'),
        ]
        appstruct = ((1, 2), (3, 4))
        result = typ.set_value(node, appstruct, 'bar.c', 34)
        self.assertEqual(result, ((1, 2), (34, 4)))

    def test_set_value_bad_path(self):
        typ = self._makeOne()
        node = DummySchemaNode(typ, name='node')
        node.children = [
            DummySchemaNode(None, name='foo'),
            DummySchemaNode(None, name='bar')
        ]
        self.assertRaises(
            KeyError, typ.set_value, node, (1, 2), 'foobar', 34)

    def test_get_value(self):
        typ = self._makeOne()
        node = DummySchemaNode(typ, name='node')
        node.children = [
            DummySchemaNode(typ, name='foo'),
            DummySchemaNode(typ, name='bar')
        ]
        node['foo'].children = [
            DummySchemaNode(None, name='a'),
            DummySchemaNode(None, name='b'),
        ]
        node['bar'].children = [
            DummySchemaNode(None, name='c'),
            DummySchemaNode(None, name='d'),
        ]
        appstruct = ((1, 2), (3, 4))
        self.assertEqual(typ.get_value(node, appstruct, 'foo'), (1, 2))
        self.assertEqual(typ.get_value(node, appstruct, 'foo.b'), 2)

    def test_get_value_bad_path(self):
        typ = self._makeOne()
        node = DummySchemaNode(typ, name='node')
        node.children = [
            DummySchemaNode(None, name='foo'),
            DummySchemaNode(None, name='bar')
        ]
        self.assertRaises(
            KeyError, typ.get_value, node, (1, 2), 'foobar')

#<removed class TestSequence(unittest.TestCase):

class TestString(unittest.TestCase):
    def _makeOne(self, encoding=None):
        from nive.components.reform.schema import String
        return String(encoding)

    def test_alias(self):
        from nive.components.reform.schema import Str
        from nive.components.reform.schema import String
        self.assertEqual(Str, String)

    def test_deserialize_emptystring(self):
        from nive.components.reform.schema import null
        node = DummySchemaNode(None)
        typ = self._makeOne(None)
        result = typ.deserialize(node, '')
        self.assertEqual(result, '')

    def test_deserialize_uncooperative(self):
        val = Uncooperative()
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, val)
        self.failUnless(e.msg)

    def test_deserialize_unicode_from_None(self):
        uni = u'\xf8'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, uni)
        self.assertEqual(result, uni)

    def test_deserialize_nonunicode_from_None(self):
        value = object()
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, value)
        self.assertEqual(result, unicode(value))

    def test_deserialize_from_utf8(self):
        uni = u'\xf8'
        utf8 = uni.encode('utf-8')
        node = DummySchemaNode(None)
        typ = self._makeOne('utf-8')
        result = typ.deserialize(node, utf8)
        self.assertEqual(result, uni)

    def test_deserialize_from_utf16(self):
        uni = u'\xf8'
        utf16 = uni.encode('utf-16')
        node = DummySchemaNode(None)
        typ = self._makeOne('utf-16')
        result = typ.deserialize(node, utf16)
        self.assertEqual(result, uni)

    def test_serialize_null(self):
        from nive.components.reform.schema import null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, null)
        self.assertEqual(result, null)

    def test_serialize_uncooperative(self):
        val = Uncooperative()
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, val)
        self.failUnless(e.msg)

    def test_serialize_nonunicode_to_None(self):
        value = object()
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, value)
        self.assertEqual(result, unicode(value))

    def test_serialize_unicode_to_None(self):
        value = u'abc'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, value)
        self.assertEqual(result, value)

    def test_serialize_to_utf8(self):
        uni = u'\xf8'
        utf8 = uni.encode('utf-8')
        node = DummySchemaNode(None)
        typ = self._makeOne('utf-8')
        result = typ.serialize(node, uni)
        self.assertEqual(result, utf8)

    def test_serialize_to_utf16(self):
        uni = u'\xf8'
        utf16 = uni.encode('utf-16')
        node = DummySchemaNode(None)
        typ = self._makeOne('utf-16')
        result = typ.serialize(node, uni)
        self.assertEqual(result, utf16)

    def test_serialize_string_with_high_unresolveable_high_order_chars(self):
        not_utf8 = '\xff\xfe\xf8\x00'
        node = DummySchemaNode(None)
        typ = self._makeOne('utf-8')
        e = invalid_exc(typ.serialize, node, not_utf8)
        self.failUnless('cannot be serialized' in e.msg)

class TestInteger(unittest.TestCase):
    def _makeOne(self):
        from nive.components.reform.schema import Integer
        return Integer()

    def test_alias(self):
        from nive.components.reform.schema import Int
        from nive.components.reform.schema import Integer
        self.assertEqual(Int, Integer)

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_serialize_emptystring(self):
        from nive.components import reform
        val = ''
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_fails(self):
        val = 'P'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, val)
        self.failUnless(e.msg)

    def test_deserialize_ok(self):
        val = '1'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, val)
        self.assertEqual(result, 1)

    def test_serialize_fails(self):
        val = 'P'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, val)
        self.failUnless(e.msg)

    def test_serialize_ok(self):
        val = 1
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, '1')

class TestFloat(unittest.TestCase):
    def _makeOne(self):
        from nive.components.reform.schema import Float
        return Float()

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_serialize_emptystring(self):
        from nive.components import reform
        val = ''
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_fails(self):
        val = 'P'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, val)
        self.failUnless(e.msg)

    def test_deserialize_ok(self):
        val = '1.0'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, val)
        self.assertEqual(result, 1.0)

    def test_serialize_fails(self):
        val = 'P'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, val)
        self.failUnless(e.msg)

    def test_serialize_ok(self):
        val = 1.0
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, '1.0')

class TestDecimal(unittest.TestCase):
    def _makeOne(self):
        from nive.components.reform.schema import Decimal
        return Decimal()

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_serialize_emptystring(self):
        from nive.components import reform
        val = ''
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_fails(self):
        val = 'P'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, val)
        self.failUnless(e.msg)

    def test_deserialize_ok(self):
        import decimal
        val = '1.0'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, val)
        self.assertEqual(result, decimal.Decimal('1.0'))

    def test_serialize_fails(self):
        val = 'P'
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, node, val)
        self.failUnless(e.msg)

    def test_serialize_ok(self):
        val = 1.0
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, '1.0')

class TestBoolean(unittest.TestCase):
    def _makeOne(self):
        from nive.components.reform.schema import Boolean
        return Boolean()

    def test_alias(self):
        from nive.components.reform.schema import Bool
        from nive.components.reform.schema import Boolean
        self.assertEqual(Bool, Boolean)

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize(self):
        typ = self._makeOne()
        node = DummySchemaNode(None)
        self.assertEqual(typ.deserialize(node, 'false'), False)
        self.assertEqual(typ.deserialize(node, 'FALSE'), False)
        self.assertEqual(typ.deserialize(node, '0'), False)
        self.assertEqual(typ.deserialize(node, 'true'), True)
        self.assertEqual(typ.deserialize(node, 'other'), True)

    def test_deserialize_unstringable(self):
        typ = self._makeOne()
        node = DummySchemaNode(None)
        e = invalid_exc(typ.deserialize, node, Uncooperative())
        self.failUnless(e.msg.endswith('not a string'))

    def test_deserialize_null(self):
        from nive.components import reform
        typ = self._makeOne()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, reform.schema.null)
        self.assertEqual(result, reform.schema.null)

    def test_serialize(self):
        typ = self._makeOne()
        node = DummySchemaNode(None)
        self.assertEqual(typ.serialize(node, 1), 'true')
        self.assertEqual(typ.serialize(node, True), 'true')
        self.assertEqual(typ.serialize(node, None), 'false')
        self.assertEqual(typ.serialize(node, False), 'false')

#<removed class TestGlobalObject(unittest.TestCase):


class TestDateTime(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from nive.components.reform.schema import DateTime
        return DateTime(*arg, **kw)

    def _dt(self):
        import datetime
        return datetime.datetime(2010, 4, 26, 10, 48)

    def _today(self):
        import datetime
        return datetime.date.today()

    def test_ctor_default_tzinfo_None(self):
        import iso8601
        typ = self._makeOne()
        self.assertEqual(typ.default_tzinfo.__class__, iso8601.iso8601.Utc)

    def test_ctor_default_tzinfo_non_None(self):
        import iso8601
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        typ = self._makeOne(default_tzinfo=tzinfo)
        self.assertEqual(typ.default_tzinfo, tzinfo)

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_serialize_with_garbage(self):
        typ = self._makeOne()
        node = DummySchemaNode(None)
        e = invalid_exc(typ.serialize, node, 'garbage')
        self.assertEqual(e.msg.interpolate(),
                         '"garbage" is not a datetime object')

    def test_serialize_with_date(self):
        import datetime
        typ = self._makeOne()
        date = self._today()
        node = DummySchemaNode(None)
        result = typ.serialize(node, date)
        expected = datetime.datetime.combine(date, datetime.time())
        expected = expected.replace(tzinfo=typ.default_tzinfo).isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_naive_datetime(self):
        typ = self._makeOne()
        node = DummySchemaNode(None)
        dt = self._dt()
        result = typ.serialize(node, dt)
        expected = dt.replace(tzinfo=typ.default_tzinfo).isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_none_tzinfo_naive_datetime(self):
        typ = self._makeOne(default_tzinfo=None)
        node = DummySchemaNode(None)
        dt = self._dt()
        result = typ.serialize(node, dt)
        self.assertEqual(result, dt.isoformat())

    def test_serialize_with_tzware_datetime(self):
        import iso8601
        typ = self._makeOne()
        dt = self._dt()
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        dt = dt.replace(tzinfo=tzinfo)
        node = DummySchemaNode(None)
        result = typ.serialize(node, dt)
        expected = dt.isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_datetime_str(self):
        import iso8601
        typ = self._makeOne()
        node = DummySchemaNode(None)
        dt = "2012-01-01T13:34:00"
        result = typ.serialize(node, dt)
        expected = iso8601.parse_date(dt).isoformat()
        self.assertEqual(result, expected)

    def test_deserialize_date(self):
        import datetime
        import iso8601
        date = self._today()
        typ = self._makeOne()
        formatted = date.isoformat()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, formatted)
        expected = datetime.datetime.combine(result, datetime.time())
        tzinfo = iso8601.iso8601.Utc()
        expected = expected.replace(tzinfo=tzinfo)
        self.assertEqual(result.isoformat(), expected.isoformat())

    def test_deserialize_invalid_ParseError(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, 'garbage')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_null(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, reform.schema.null)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_empty(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, '')
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_success(self):
        import iso8601
        typ = self._makeOne()
        dt = self._dt()
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        dt = dt.replace(tzinfo=tzinfo)
        iso = dt.isoformat()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, iso)
        self.assertEqual(result.isoformat(), iso)

    def test_deserialize_naive_with_default_tzinfo(self):
        import iso8601
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        typ = self._makeOne(default_tzinfo=tzinfo)
        dt = self._dt()
        dt_with_tz = dt.replace(tzinfo=tzinfo)
        iso = dt.isoformat()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, iso)
        self.assertEqual(result.isoformat(), dt_with_tz.isoformat())

    def test_deserialize_none_tzinfo(self):
        typ = self._makeOne(default_tzinfo=None)
        dt = self._dt()
        iso = dt.isoformat()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, iso)
        self.assertEqual(result.isoformat(), dt.isoformat())

class TestDate(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from nive.components.reform.schema import Date
        return Date(*arg, **kw)

    def _dt(self):
        import datetime
        return datetime.datetime(2010, 4, 26, 10, 48)

    def _today(self):
        import datetime
        return datetime.date.today()

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_serialize_with_garbage(self):
        typ = self._makeOne()
        node = DummySchemaNode(None)
        e = invalid_exc(typ.serialize, node, 'garbage')
        self.assertEqual(e.msg.interpolate(), '"garbage" is not a date object')

    def test_serialize_with_date(self):
        typ = self._makeOne()
        date = self._today()
        node = DummySchemaNode(None)
        result = typ.serialize(node, date)
        expected = date.isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_datetime(self):
        typ = self._makeOne()
        dt = self._dt()
        node = DummySchemaNode(None)
        result = typ.serialize(node, dt)
        expected = dt.date().isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_date_str(self):
        import iso8601
        typ = self._makeOne()
        date = "2012-01-01"
        node = DummySchemaNode(None)
        result = typ.serialize(node, date)
        self.assertEqual(result, date)

    def test_deserialize_invalid_ParseError(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, 'garbage')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_invalid_weird(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, '10-10-10-10')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_null(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, reform.schema.null)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_empty(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, '')
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_success_date(self):
        typ = self._makeOne()
        date = self._today()
        iso = date.isoformat()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, iso)
        self.assertEqual(result.isoformat(), iso)

    def test_deserialize_success_datetime(self):
        dt = self._dt()
        typ = self._makeOne()
        iso = dt.isoformat()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, iso)
        self.assertEqual(result.isoformat(), dt.date().isoformat())

class TestTime(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from nive.components.reform.schema import Time
        return Time(*arg, **kw)

    def _dt(self):
        import datetime
        return datetime.datetime(2010, 4, 26, 10, 48)

    def _now(self):
        import datetime
        return datetime.datetime.now().time()

    def test_serialize_null(self):
        from nive.components import reform
        val = reform.schema.null
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.serialize(node, val)
        self.assertEqual(result, reform.schema.null)

    def test_serialize_with_garbage(self):
        typ = self._makeOne()
        node = DummySchemaNode(None)
        e = invalid_exc(typ.serialize, node, 'garbage')
        self.assertEqual(e.msg.interpolate(), '"garbage" is not a time object')

    def test_serialize_with_time(self):
        typ = self._makeOne()
        time = self._now()
        node = DummySchemaNode(None)
        result = typ.serialize(node, time)
        expected = time.isoformat().split('.')[0]
        self.assertEqual(result, expected)

    def test_serialize_with_datetime(self):
        typ = self._makeOne()
        dt = self._dt()
        node = DummySchemaNode(None)
        result = typ.serialize(node, dt)
        expected = dt.time().isoformat().split('.')[0]
        self.assertEqual(result, expected)

    def test_serialize_with_time_str(self):
        typ = self._makeOne()
        time = "12:34:00"
        node = DummySchemaNode(None)
        result = typ.serialize(node, time)
        self.assertEqual(result, time)

    def test_deserialize_invalid_ParseError(self):
        node = DummySchemaNode(None)
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, 'garbage')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_three_digit_string(self):
        import datetime
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, '11:00:11')
        self.assertEqual(result, datetime.time(11, 0, 11))

    def test_deserialize_two_digit_string(self):
        import datetime
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, '11:00')
        self.assertEqual(result, datetime.time(11, 0))

    def test_deserialize_null(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, reform.schema.null)
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_empty(self):
        from nive.components import reform
        node = DummySchemaNode(None)
        typ = self._makeOne()
        result = typ.deserialize(node, '')
        self.assertEqual(result, reform.schema.null)

    def test_deserialize_missing_seconds(self):
        import datetime
        typ = self._makeOne()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, '10:12')
        self.assertEqual(result, datetime.time(10, 12))

    def test_deserialize_success_time(self):
        import datetime
        typ = self._makeOne()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, '10:12:13')
        self.assertEqual(result, datetime.time(10, 12, 13))

    def test_deserialize_success_datetime(self):
        dt = self._dt()
        typ = self._makeOne()
        iso = dt.isoformat()
        node = DummySchemaNode(None)
        result = typ.deserialize(node, iso)
        self.assertEqual(result.isoformat(),
                dt.time().isoformat().split('.')[0])

class TestSchemaNode(unittest.TestCase):
    def _makeOne(self, *arg, **kw):
        from nive.components.reform.schema import SchemaNode
        return SchemaNode(*arg, **kw)

    def test_new_sets_order(self):
        node = self._makeOne(None)
        self.failUnless(hasattr(node, '_order'))

    def test_ctor_no_title(self):
        node = self._makeOne(None, 0, validator=1, default=2, name='name_a',
                             missing='missing')
        self.assertEqual(node.typ, None)
        self.assertEqual(node.children, [0])
        self.assertEqual(node.validator, 1)
        self.assertEqual(node.default, 2)
        self.assertEqual(node.missing, 'missing')
        self.assertEqual(node.name, 'name_a')
        self.assertEqual(node.title, 'Name A')

    def test_ctor_with_title(self):
        node = self._makeOne(None, 0, validator=1, default=2, name='name',
                             title='title')
        self.assertEqual(node.typ, None)
        self.assertEqual(node.children, [0])
        self.assertEqual(node.validator, 1)
        self.assertEqual(node.default, 2)
        self.assertEqual(node.name, 'name')
        self.assertEqual(node.title, 'title')

    def test_ctor_with_description(self):
        node = self._makeOne(None, 0, validator=1, default=2, name='name',
                             title='title', description='desc')
        self.assertEqual(node.description, 'desc')

    def test_ctor_with_widget(self):
        node = self._makeOne(None, 0, widget='abc')
        self.assertEqual(node.widget, 'abc')

    def test_ctor_with_preparer(self):
        node = self._makeOne(None, 0, preparer='abc')
        self.assertEqual(node.preparer, 'abc')

    def test_ctor_without_preparer(self):
        node = self._makeOne(None, 0)
        self.assertEqual(node.preparer, None)

    def test_ctor_with_unknown_kwarg(self):
        node = self._makeOne(None, 0, foo=1)
        self.assertEqual(node.foo, 1)

    def test_required_true(self):
        node = self._makeOne(None)
        self.assertEqual(node.required, True)

    def test_required_false(self):
        node = self._makeOne(None, missing=1)
        self.assertEqual(node.required, False)

    def test_required_deferred(self):
        from nive.components.reform.schema import deferred
        node = self._makeOne(None, missing=deferred('123'))
        self.assertEqual(node.required, True)

    def test_deserialize_no_validator(self):
        typ = DummyType()
        node = self._makeOne(typ)
        result = node.deserialize(1)
        self.assertEqual(result, 1)

    def test_deserialize_with_preparer(self):
        from nive.components.reform.schema import Invalid
        typ = DummyType()
        def preparer(value):
            return 'prepared_'+value
        def validator(node, value):
            if not value.startswith('prepared'):
                raise Invalid(node, 'not prepared') # pragma: no cover
        node = self._makeOne(typ,
                             preparer=preparer,
                             validator=validator)
        self.assertEqual(node.deserialize('value'),
                         'prepared_value')

    def test_deserialize_preparer_before_missing_check(self):
        from nive.components.reform.schema import null
        typ = DummyType()
        def preparer(value):
            return null
        node = self._makeOne(typ,preparer=preparer)
        e = invalid_exc(node.deserialize, 1)
        self.assertEqual(e.msg, 'Required')

    def test_deserialize_with_validator(self):
        typ = DummyType()
        validator = DummyValidator(msg='Wrong')
        node = self._makeOne(typ, validator=validator)
        e = invalid_exc(node.deserialize, 1)
        self.assertEqual(e.msg, 'Wrong')

    def test_deserialize_value_is_null_no_missing(self):
        from nive.components.reform.schema import null
        from nive.components.reform.schema import Invalid
        typ = DummyType()
        node = self._makeOne(typ)
        self.assertRaises(Invalid, node.deserialize, null)

    def test_deserialize_value_is_null_with_missing(self):
        from nive.components.reform.schema import null
        typ = DummyType()
        node = self._makeOne(typ)
        node.missing = 'abc'
        self.assertEqual(node.deserialize(null), 'abc')

    def test_deserialize_noargs_uses_default(self):
        typ = DummyType()
        node = self._makeOne(typ)
        node.missing = 'abc'
        self.assertEqual(node.deserialize(), 'abc')

    def test_deserialize_null_can_be_used_as_missing(self):
        from nive.components.reform.schema import null
        typ = DummyType()
        node = self._makeOne(typ)
        node.missing = null
        self.assertEqual(node.deserialize(null), null)

    def test_deserialize_appstruct_deferred(self):
        from nive.components.reform.schema import null
        from nive.components.reform.schema import deferred
        from nive.components.reform.schema import Invalid
        typ = DummyType()
        node = self._makeOne(typ)
        node.missing = deferred('123')
        self.assertRaises(Invalid, node.deserialize, null)

    def test_serialize(self):
        typ = DummyType()
        node = self._makeOne(typ)
        result = node.serialize(1)
        self.assertEqual(result, 1)

    def test_serialize_value_is_null_no_default(self):
        from nive.components.reform.schema import null
        typ = DummyType()
        node = self._makeOne(typ)
        result = node.serialize(null)
        self.assertEqual(result, null)

    def test_serialize_value_is_null_with_default(self):
        from nive.components.reform.schema import null
        typ = DummyType()
        node = self._makeOne(typ)
        node.default = 1
        result = node.serialize(null)
        self.assertEqual(result, 1)

    def test_serialize_noargs_uses_default(self):
        typ = DummyType()
        node = self._makeOne(typ)
        node.default = 'abc'
        self.assertEqual(node.serialize(), 'abc')

    def test_serialize_default_deferred(self):
        from nive.components.reform.schema import deferred
        from nive.components.reform.schema import null
        typ = DummyType()
        node = self._makeOne(typ)
        node.default = deferred('abc')
        self.assertEqual(node.serialize(), null)

    def test_add(self):
        node = self._makeOne(None)
        node.add(1)
        self.assertEqual(node.children, [1])

    def test_repr(self):
        node = self._makeOne(None, name='flub')
        result = repr(node)
        self.failUnless(result.startswith('<nive.components.reform.schema.SchemaNode object at '))
        self.failUnless(result.endswith("(named flub)>"))

    def test___getitem__success(self):
        node = self._makeOne(None)
        another = self._makeOne(None, name='another')
        node.add(another)
        self.assertEqual(node['another'], another)

    def test___getitem__failure(self):
        node = self._makeOne(None)
        self.assertRaises(KeyError, node.__getitem__, 'another')

    def test___delitem__success(self):
        node = self._makeOne(None)
        another = self._makeOne(None, name='another')
        node.add(another)
        del node['another']
        self.assertEqual(node.children, [])

    def test___delitem__failure(self):
        node = self._makeOne(None)
        self.assertRaises(KeyError, node.__delitem__, 'another')

    def test___setitem__success(self):
        node = self._makeOne(None)
        another = self._makeOne(None, name='another')
        node.add(another)
        andanother = self._makeOne(None, name='andanother')
        node['another'] = andanother
        self.assertEqual(node['another'], andanother)
        self.assertEqual(andanother.name, 'another')

    def test___setitem__failure(self):
        node = self._makeOne(None)
        self.assertRaises(KeyError, node.__setitem__, 'another', None)

    def test___iter__(self):
        node = self._makeOne(None)
        node.children = ['a', 'b', 'c']
        it = node.__iter__()
        self.assertEqual(list(it), ['a', 'b', 'c'])

    def test___contains__(self):
        node = self._makeOne(None)
        another = self._makeOne(None, name='another')
        node.add(another)
        self.assertEquals('another' in node, True)
        self.assertEquals('b' in node, False)

    def test_clone(self):
        inner_typ = DummyType()
        outer_typ = DummyType()
        outer_node = self._makeOne(outer_typ, name='outer')
        inner_node = self._makeOne(inner_typ, name='inner')
        outer_node.foo = 1
        inner_node.foo = 2
        outer_node.children = [inner_node]
        outer_clone = outer_node.clone()
        self.failIf(outer_clone is outer_node)
        self.assertEqual(outer_clone.typ, outer_typ)
        self.assertEqual(outer_clone.name, 'outer')
        self.assertEqual(outer_node.foo, 1)
        self.assertEqual(len(outer_clone.children), 1)
        inner_clone = outer_clone.children[0]
        self.failIf(inner_clone is inner_node)
        self.assertEqual(inner_clone.typ, inner_typ)
        self.assertEqual(inner_clone.name, 'inner')
        self.assertEqual(inner_clone.foo, 2)

    def test_bind(self):
        from nive.components.reform.schema import deferred
        inner_typ = DummyType()
        outer_typ = DummyType()
        def dv(node, kw):
            self.failUnless(node.name in ['outer', 'inner'])
            self.failUnless('a' in kw)
            return '123'
        dv = deferred(dv)
        outer_node = self._makeOne(outer_typ, name='outer', missing=dv)
        inner_node = self._makeOne(inner_typ, name='inner', validator=dv,
                                   missing=dv)
        outer_node.children = [inner_node]
        outer_clone = outer_node.bind(a=1)
        self.failIf(outer_clone is outer_node)
        self.assertEqual(outer_clone.missing, '123')
        inner_clone = outer_clone.children[0]
        self.failIf(inner_clone is inner_node)
        self.assertEqual(inner_clone.missing, '123')
        self.assertEqual(inner_clone.validator, '123')

    def test_bind_with_after_bind(self):
        from nive.components.reform.schema import deferred
        inner_typ = DummyType()
        outer_typ = DummyType()
        def dv(node, kw):
            self.failUnless(node.name in ['outer', 'inner'])
            self.failUnless('a' in kw)
            return '123'
        dv = deferred(dv)
        def remove_inner(node, kw):
            self.assertEqual(kw, {'a':1})
            del node['inner']
        outer_node = self._makeOne(outer_typ, name='outer', missing=dv,
                                   after_bind=remove_inner)
        inner_node = self._makeOne(inner_typ, name='inner', validator=dv,
                                   missing=dv)
        outer_node.children = [inner_node]
        outer_clone = outer_node.bind(a=1)
        self.failIf(outer_clone is outer_node)
        self.assertEqual(outer_clone.missing, '123')
        self.assertEqual(len(outer_clone.children), 0)
        self.assertEqual(len(outer_node.children), 1)

class TestDeferred(unittest.TestCase):
    def _makeOne(self, wrapped):
        from nive.components.reform.schema import deferred
        return deferred(wrapped)

    def test_ctor(self):
        wrapped = '123'
        inst = self._makeOne(wrapped)
        self.assertEqual(inst.wrapped, wrapped)

    def test___call__(self):
        n = object()
        k = object()
        def wrapped(node, kw):
            self.assertEqual(node, n)
            self.assertEqual(kw, k)
            return 'abc'
        inst = self._makeOne(wrapped)
        result= inst(n, k)
        self.assertEqual(result, 'abc')

class TestSchema(unittest.TestCase):
    def test_alias(self):
        from nive.components.reform.schema import Schema
        from nive.components.reform.schema import MappingSchema
        self.assertEqual(Schema, MappingSchema)

    def test_it(self):
        from nive.components import reform
        class MySchema(reform.schema.Schema):
            thing_a = reform.schema.SchemaNode(reform.schema.String())
            thing2 = reform.schema.SchemaNode(reform.schema.String(), title='bar')
        node = MySchema(default='abc')
        self.failUnless(hasattr(node, '_order'))
        self.assertEqual(node.default, 'abc')
        self.assertEqual(node.__class__, reform.schema.SchemaNode)
        self.assertEqual(node.typ.__class__, reform.schema.Mapping)
        self.assertEqual(node.children[0].typ.__class__, reform.schema.String)
        self.assertEqual(node.children[0].title, 'Thing A')
        self.assertEqual(node.children[1].title, 'bar')

    def test_title_munging(self):
        from nive.components import reform
        class MySchema(reform.schema.Schema):
            thing1 = reform.schema.SchemaNode(reform.schema.String())
            thing2 = reform.schema.SchemaNode(reform.schema.String(), title=None)
            thing3 = reform.schema.SchemaNode(reform.schema.String(), title='')
            thing4 = reform.schema.SchemaNode(reform.schema.String(), title='thing2')
        node = MySchema()
        self.assertEqual(node.children[0].title, 'Thing1')
        self.assertEqual(node.children[1].title, None)
        self.assertEqual(node.children[2].title, '')
        self.assertEqual(node.children[3].title, 'thing2')

#<removed class TestSequenceSchema(unittest.TestCase):
#<removed class TestTupleSchema(unittest.TestCase):
#<removed class TestFunctional(object):
#<removed class TestImperative(unittest.TestCase, TestFunctional):
#<removed class TestDeclarative(unittest.TestCase, TestFunctional):

class Test_null(unittest.TestCase):
    def test___nonzero__(self):
        from nive.components.reform.schema import null
        self.failIf(null)

    def test___repr__(self):
        from nive.components.reform.schema import null
        self.assertEqual(repr(null), '<schema.null>')

    def test_pickling(self):
        from nive.components.reform.schema import null
        import cPickle
        self.failUnless(cPickle.loads(cPickle.dumps(null)) is null)

class Dummy(object):
    pass

class DummySchemaNode(object):
    def __init__(self, typ, name='', exc=None, default=None):
        self.typ = typ
        self.name = name
        self.exc = exc
        self.required = default is None
        self.default = default
        self.children = []

    def deserialize(self, val, formstruct=None):
        from nive.components.reform.schema import Invalid
        if self.exc:
            raise Invalid(self, self.exc)
        return val

    def serialize(self, val):
        from nive.components.reform.schema import Invalid
        if self.exc:
            raise Invalid(self, self.exc)
        return val

    def __getitem__(self, name):
        for child in self.children:
            if child.name == name:
                return child

class DummyValidator(object):
    def __init__(self, msg=None):
        self.msg = msg

    def __call__(self, node, value):
        from nive.components.reform.schema import Invalid
        if self.msg:
            raise Invalid(node, self.msg)

class Uncooperative(object):
    def __str__(self):
        raise ValueError('I wont cooperate')

    __unicode__ = __str__

class DummyType(object):
    def serialize(self, node, value):
        return value

    def deserialize(self, node, value, formstruct=None):
        return value

    def flatten(self, node, appstruct, prefix='', listitem=False):
        if listitem:
            key = prefix.rstrip('.')
        else:
            key = prefix + 'appstruct'
        return {key:appstruct}

    def unflatten(self, node, paths, fstruct):
        assert paths == [node.name]
        return fstruct[node.name]

class TestSet(unittest.TestCase):
    def _makeOne(self, **kw):
        from nive.components.reform.schema import Set
        return Set(**kw)

    def test_serialize(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        provided = []
        result = typ.serialize(node, provided)
        self.failUnless(result is provided)

    def test_serialize_null(self):
        from nive.components.reform.schema import null
        node = DummySchemaNode2()
        typ = self._makeOne()
        result = typ.serialize(node, null)
        self.assertEqual(result, null)

    def test_deserialize_no_iter(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, 'str')
        self.assertEqual(e.msg, '${value} is not iterable')

    def test_deserialize_null(self):
        from nive.components.reform.schema import null
        node = DummySchemaNode2()
        typ = self._makeOne()
        result = typ.deserialize(node, null)
        self.assertEqual(result, null)

    def test_deserialize_valid(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        result = typ.deserialize(node, ('a',))
        self.assertEqual(result, set(('a',)))

    def test_deserialize_empty_allow_empty_false(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, ())
        self.assertEqual(e.msg, 'Required')

    def test_deserialize_empty_allow_empty_true(self):
        node = DummySchemaNode2()
        typ = self._makeOne(allow_empty=True)
        result = typ.deserialize(node, ())
        self.assertEqual(result, set())

class TestFileData(unittest.TestCase):
    def _makeOne(self):
        from nive.components.reform.schema import FileData
        return FileData()

    def test_deserialize_null(self):
        from nive.components.reform.schema import null
        typ = self._makeOne()
        node = DummySchemaNode2()
        result = typ.deserialize(node, null)
        self.assertEqual(result, null)

    def test_deserialize_not_null(self):
        typ = self._makeOne()
        node = DummySchemaNode2()
        result = typ.deserialize(node, '123')
        self.assertEqual(result, '123')

    def test_serialize_null(self):
        from nive.components.reform.schema import null
        typ = self._makeOne()
        node = DummySchemaNode2()
        result = typ.serialize(node, null)
        self.assertEqual(result, null)

    def test_serialize_no_filename(self):
        typ = self._makeOne()
        node = DummySchemaNode2()
        e = invalid_exc(typ.serialize, node, {'uid':'uid'})
        self.assertEqual(e.msg, "${value} has no ${key} key")

    def test_serialize_no_uid(self):
        typ = self._makeOne()
        node = DummySchemaNode2()
        e = invalid_exc(typ.serialize, node, {'filename':'filename'})
        self.assertEqual(e.msg, "${value} has no ${key} key")

    def test_serialize_with_values(self):
        typ = self._makeOne()
        node = DummySchemaNode2()
        result = typ.serialize(node, {'filename':'filename', 'uid':'uid',
                                      'mimetype':'mimetype', 'size':'size',
                                      'fp':'fp', 'preview_url':'preview_url'})
        self.assertEqual(result['filename'], 'filename')
        self.assertEqual(result['uid'], 'uid')
        self.assertEqual(result['mimetype'], 'mimetype')
        self.assertEqual(result['size'], 'size')
        self.assertEqual(result['fp'], 'fp')
        self.assertEqual(result['preview_url'], 'preview_url')
        

class TestList(unittest.TestCase):
    def _makeOne(self, **kw):
        from nive.components.reform.schema import List
        return List(**kw)

    def test_serialize(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        provided = []
        result = typ.serialize(node, provided)
        self.failUnless(result is provided)

    def test_serialize_null(self):
        from nive.components.reform.schema import null
        node = DummySchemaNode2()
        typ = self._makeOne()
        result = typ.serialize(node, null)
        self.assertEqual(result, null)

    def test_deserialize_no_iter(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, 123)
        self.assertEqual(e.msg, '${value} is not iterable')

    def test_deserialize_null(self):
        from nive.components.reform.schema import null
        node = DummySchemaNode2()
        typ = self._makeOne()
        result = typ.deserialize(node, null)
        self.assertEqual(result, null)

    def test_deserialize_valid(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        result = typ.deserialize(node, ('a',))
        self.assertEqual(result, ['a'])

        node = DummySchemaNode2()
        typ = self._makeOne()
        result = typ.deserialize(node, 'str')
        self.assertEqual(result, ['str'])

    def test_deserialize_empty_allow_empty_false(self):
        node = DummySchemaNode2()
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, node, ())
        self.assertEqual(e.msg, 'Required')

    def test_deserialize_empty_allow_empty_true(self):
        node = DummySchemaNode2()
        typ = self._makeOne(allow_empty=True)
        result = typ.deserialize(node, ())
        self.assertEqual(result, [])
        
        
class DummySchemaNode2(object):
    def __init__(self, typ=None, name='', exc=None, default=None):
        self.typ = typ
        self.name = name
        self.exc = exc
        self.required = default is None
        self.default = default
        self.children = []





from nive.definitions import ModuleConf

configuration = ModuleConf(
    id="cms-admin-module",
    name="Meta package to load cms admin ui components",
    modules=(
        "nive.adminview.view", 
        "nive.components.reform.reformed"
    ),
)


"""
Running mysql tests
-------------------
Create a database 'ut_nive' and assign all permissions for user 'root@localhost'. 
File root is '/var/tmp/nive'.

To customize settings change 'dbconfMySql' in this file and 'conn' in 
nive/utils/dataPool2/tests/t_MySql.py.  

"""


import time
import unittest
from StringIO import StringIO

from nive.utils.path import DvPath

from nive.definitions import *
from nive.security import User
from nive.components.objects.base import ApplicationBase

from db_app import *
from test_nive import appTest_db
from test_container import containerTest_db, groupsrootTest_db
from test_objects import objTest_db, objToolTest_db, objWfTest_db, groupsTest_db

from nive.tests import __local

# real database test configuration
# change these to fit your system
ENABLE_MYSQL_TESTS = True
try:
    import MySQLdb
except ImportError:
    ENABLE_MYSQL_TESTS = False

dbconfMySql = DatabaseConf(
    context = "MySql",
    dbName = __local.DATABASE,
    fileRoot = __local.ROOT,
    host = __local.HOST,
    user = __local.USER,
    port = __local.PORT,
    password = __local.PASSWORD
)

if not ENABLE_MYSQL_TESTS:
    class utc:
        pass
    uTestCase = utc
else:
    uTestCase = unittest.TestCase
    
    
def myapp(modules=None):
    a = ApplicationBase()
    a.Register(appconf)
    a.Register(dbconfMySql)
    if modules:
        for m in modules:
            a.Register(m)
    p = Portal()
    p.Register(a, "nive")
    a.LoadConfiguration()
    root = DvPath(a.dbConfiguration.fileRoot)
    if not root.IsDirectory():
        root.CreateDirectories()
    try:
        a.Query("select id from pool_meta where id=1")
        a.Query("select id from data1 where id=1")
        a.Query("select id from data2 where id=1")
        a.Query("select id from data3 where id=1")
        a.Query("select id from pool_files where id=1")
        a.Query("select id from pool_sys where id=1")
        a.Query("select id from pool_groups where id=1")
    except:
        a.GetTool("nive.components.tools.dbStructureUpdater")()
    a.Startup(None)
    return a


class myappTest_db(appTest_db, uTestCase):

    def setUp(self):
        #emptypool()
        self.app = myapp()
        self.remove=[]


class mycontainerTest_db(containerTest_db, uTestCase):

    def setUp(self):
        #emptypool()
        self.app = myapp()
        self.remove=[]

class mygroupsTest_db(groupsTest_db, uTestCase):

    def setUp(self):
        #emptypool()
        self.app = myapp(["nive.components.extensions.localgroups"])
        self.remove=[]

class myobjTest_db(objTest_db, uTestCase):

    def setUp(self):
        #emptypool()
        self.app = myapp()
        self.remove=[]


class mygroupsrootTest_db(groupsrootTest_db, uTestCase):

    def setUp(self):
        #emptypool()
        self.app = myapp(["nive.components.extensions.localgroups"])
        self.remove=[]



#tests!
#class myobjToolTest_db(objToolTest_db, uTestCase):
#
#    def setUp(self):
#        #emptypool()
#        self.app = myapp()

#class myobjWfTest_db(objWfTest_db, uTestCase):
#
#    def setUp(self):
#        #emptypool()
#        self.app = myapp()



if __name__ == '__main__':
    unittest.main()



